/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <gtc_main.h>

namespace gtc
{

  using namespace std;
  using namespace sstmac;
  using namespace sstmac::sw;

  /**
   * ---------------------------------------------------------------------------------------
   * smooth
   * @param something
   * ---------------------------------------------------------------------------------------
   */
  void
  gtc_main::smooth(int iflag)
  {

    //short names
    field_array::ptr fa = field_array_;
    particle_decomp::ptr pd = part_decomp_;
    config_parameters::ptr p = gtcparams_;


    SSTMAC_DEBUG << "gtc(" << p->mype_ << "):: starting smooth(" << iflag << ") \n";

    double scale = 1.0;

    if (iflag == 0)
      {
        //$omp parallel do private(i)
        comp_->compute_loop2(1, fa->mgrid_, 1, pd->mzeta_, 1);

      }
    else if (iflag == 1)
      {
        //$omp parallel do private(i)
        comp_->compute_loop2(1, fa->mgrid_, 1, pd->mzeta_, 1);

      }
    else
      {
        //$omp parallel do private(i)
        comp_->compute_loop2(1, fa->mgrid_, 1, pd->mzeta_, 1);

      }

    int ismooth = 1;
    if (p->nonlinear_ < 0.5)
      {
        ismooth = 0;
      }

    for (int ip = 1; ip <= ismooth; ip++)
      {
        //radial smoothing
        comp_->compute_loop2(1, p->mpsi_, 1, pd->mzeta_, 1);

        comp_->compute_loop2(1, fa->mgrid_, 1, pd->mzeta_ + 1, 4);

        // poloidal smoothing (-0.0625 0.25 0.625 0.25 -0.0625)
        //$omp parallel do private(i,j,ii,jt,pright)
        comp_->compute_loop(1, fa->mgrid_, 2);

        // parallel smoothing
        // send phi to right and receive from left
        comp_->compute_loop(1, fa->mgrid_, 1);

        int icount = fa->mgrid_;
        int idest = pd->right_pe_;
        int isource = pd->left_pe_;
        int isendtag = pd->myrank_toroidal_;
        int irecvtag = isource;
        MPI_Status istatus;

        MPI_SendRecv(NULL, icount, MPI_DOUBLE, idest, isendtag, NULL, icount,
            MPI_DOUBLE, isource, irecvtag, pd->toroidal_comm_, &istatus);

        // send phi to left and receive from right
        comp_->compute_loop(1, fa->mgrid_, 1);

        idest = pd->left_pe_;
        isource = pd->right_pe_;
        isendtag = pd->myrank_toroidal_;
        irecvtag = isource;

        MPI_SendRecv(NULL, icount, MPI_DOUBLE, idest, isendtag, NULL, icount,
            MPI_DOUBLE, isource, irecvtag, pd->toroidal_comm_, &istatus);

        //$omp parallel do private(i,ii,jt,j,ij,ptemp,pleft,pright)
        comp_->compute_loop(1, fa->mgrid_, 3);
        comp_->compute_loop2(1, fa->mgrid_, 1, pd->mzeta_, 3);

      }

    // toroidal BC: send phi to right and receive from left
    comp_->compute_loop(1, fa->mgrid_, 1);

    int icount = fa->mgrid_;
    int idest = pd->right_pe_;
    int isource = pd->left_pe_;
    int isendtag = pd->myrank_toroidal_;
    int irecvtag = isource;
    MPI_Status istatus;

    MPI_SendRecv(NULL, icount, MPI_DOUBLE, idest, isendtag, NULL, icount,
        MPI_DOUBLE, isource, irecvtag, pd->toroidal_comm_, &istatus);

    if (pd->myrank_toroidal_ == 0)
      {
        comp_->compute_loop(1, fa->mgrid_, 1);

      }
    else
      {
        comp_->compute_loop(1, fa->mgrid_, 1);

      }

    // poloidal BC
    comp_->compute_loop2(1, p->mpsi_, 1, pd->mzeta_, 1);

    // radial boundary
    comp_->compute_loop2(1, p->mpsi_, 1, pd->mzeta_, 1);

    if (iflag == 0)
      {
        //$omp parallel do private(i)
        comp_->compute_loop2(1, fa->mgrid_, 1, pd->mzeta_, 1);

      }
    else if (iflag == 1)
      {
        //$omp parallel do private(i)
        comp_->compute_loop2(1, fa->mgrid_, 1, pd->mzeta_, 1);

      }
    else
      {
        //$omp parallel do private(i)
        comp_->compute_loop2(1, fa->mgrid_, 1, pd->mzeta_, 1);

      }

    // solve zonal flow: phi00=r*E_r, E_r(a0)=0. Trapezoid rule
    if (iflag == 3)
      {
        if (p->nhybrid_ == 0)
          {
            comp_->compute_loop(1, p->mpsi_, 1);

          }
        if (p->nhybrid_ > 0)
          {
            comp_->compute_loop(1, p->mpsi_, 1);

          }
        // (-0.0625 0.25 0.625 0.25 -0.0625) radial smoothing of (0,0) mode density phip00

        comp_->compute_loop(1, p->mpsi_, 1);

        comp_->compute_loop(1, p->mpsi_, 2);

        comp_->compute_loop(1, p->mpsi_, 2);

        // subtract net momentum
        // phip00=phip00-sum(phip00)/real(mpsi+1)

        // d phi/dr, in equilibrium unit
        comp_->compute_loop(1, p->mpsi_, 2);

        // add FLR contribution using Pade approximation: b*<phi>=(1+b)*<n>
        comp_->compute_loop(1, p->mpsi_, 1);

        comp_->compute_loop(1, p->mpsi_, 1);

        // (0,0) mode potential store in phi00
        comp_->compute_loop(1, p->mpsi_, 1);

      }

    // Interpolate on a flux surface from fieldline coordinates to magnetic
    // coordinates. Use mtdiag for both poloidal and toroidal grid points.
    if (iflag > 1)
      {
        if (p->nonlinear_ < 0.5 || (iflag == 3 && p->idiag_ == 0))
          {

            int mzbig = max(1, p->mtdiag_ / p->mzetamax_);
            int mzmax = p->mzetamax_ * mzbig;
            int mz = pd->mzeta_ * mzbig;
            int meachtheta = p->mtdiag_ / pd->ntoroidal_;
            int icount = meachtheta * mz * (p->idiag2_ - p->idiag1_ + 1);

            comp_->compute_loop4(1, pd->mzeta_, 1, mzbig, p->idiag1_,
                p->idiag2_, 1, p->mtdiag_, 6);

            // transpose 2-d matrix from (ntoroidal,mzeta*mzbig) to (1,mzetamax*mzbig)
            for (int jpe = 0; jpe <= pd->ntoroidal_ - 1; jpe++)
              {

                // $omp parallel do private(j,i,k,jt,indt,indp1,indp)
                comp_->compute_loop3(1, meachtheta, p->idiag1_, p->idiag2_, 1,
                    mz, 2);

                MPI_Gather(NULL, icount, MPI_DOUBLE, NULL, icount, MPI_DOUBLE,
                    jpe, pd->toroidal_comm_);

              }

            // transform to k space
            //$omp parallel do private(j,i,kz,k,indt1,indt,indp,xz,yz) firstprivate(aux2f,aux3f,aux2b,aux3b)
            for (int j = 1; j <= meachtheta; j++)
              {
                int indt1 = (j - 1) * mz;
                for (int i = p->idiag1_; i <= p->idiag2_; i++)
                  {

                    comp_->compute_loop2(1, pd->ntoroidal_, 1, mz, 2);

                    comp_->compute_fft();

                    if (p->nonlinear_ < 0.5)
                      {
                        // linear run only keep a few modes
                        comp_->compute_loop(1, p->mtdiag_, 1);

                        comp_->compute_fft();

                        // transpose back to (ntoroidal,mz)
                        comp_->compute_loop2(1, pd->ntoroidal_, 1, mz, 2);

                      }
                  }
              }

            if (p->nonlinear_ < 0.5)
              {
                for (int jpe = 0; jpe <= pd->ntoroidal_ - 1; jpe++)
                  {

                    MPI_Scatter(NULL, icount, MPI_DOUBLE, NULL, icount, MPI_DOUBLE,
                        jpe, pd->toroidal_comm_);

                    //$omp parallel do private(j,i,k,jt,indt,indp1,indp)
                    comp_->compute_loop3(1, meachtheta, p->idiag1_, p->idiag2_,
                        1, mz, 2);

                  }

                // interpolate field from magnetic coordinates to fieldline coordinates
                comp_->compute_loop3(1, pd->mzeta_, p->idiag1_, p->idiag2_, 1,
                    p->mthetamax_, 7);

                // toroidal BC: send phi to right and receive from left
                comp_->compute_loop(1, fa->mgrid_, 1);

                icount = fa->mgrid_;
                idest = pd->right_pe_;
                isource = pd->left_pe_;
                isendtag = pd->myrank_toroidal_;
                irecvtag = isource;

                MPI_SendRecv(NULL, icount, MPI_DOUBLE, idest, isendtag, NULL,
                    icount, MPI_DOUBLE, isource, irecvtag, pd->toroidal_comm_,
                    &istatus);

                if (pd->myrank_toroidal_ == 0)
                  {
                    comp_->compute_loop2(p->idiag1_, p->idiag2_, 1,
                        p->mthetamax_, 1);

                  }
                else
                  {
                    comp_->compute_loop(1, fa->mgrid_, 1);

                  }

                // poloidal BC
                comp_->compute_loop2(p->idiag1_, p->idiag2_, 1, pd->mzeta_, 1);

              }
          }
      }

    if (iflag == 2)
      {
        int j = p->irk_ + 2 * (p->ihybrid_ - 1);

        // $omp parallel do private(i)
        comp_->compute_loop2(1, fa->mgrid_, 0, pd->mzeta_, 1);

      }

    // diagnostic
    if (iflag == 3 && p->idiag_ == 0)
      {

        comp_->compute_loop(0, p->mpsi_, 1);

        //!!!efield=sqrt(efield/real(mzeta*sum(mtheta)))/(gyroradius*gyroradius)
        comp_->compute_fft();

        comp_->compute_loop(1, p->num_mode_, 2);

        // ! dominant (n,m) mode history data for flux surface at i=mpsi/2
        int meachtheta = p->mtdiag_ / pd->ntoroidal_;
        icount = meachtheta * p->num_mode_;

        MPI_Gather(NULL, 2 * icount, MPI_DOUBLE, NULL,  2 * icount, MPI_DOUBLE, 0,
            pd->toroidal_comm_);

        if (pd->myrank_toroidal_ == 0)
          {

            comp_->compute_loop3(1, p->num_mode_, 0, pd->ntoroidal_ - 1, 1,
                meachtheta, 1);
            comp_->compute_fft();

          }
      }

    SSTMAC_DEBUG << "gtc(" << p->mype_ << "):: done with smooth() \n";
  }

  /**
   * ---------------------------------------------------------------------------------------
   * field
   * ---------------------------------------------------------------------------------------
   */
  void
  gtc_main::field()
  {
    //short names
    field_array::ptr fa = field_array_;
    particle_decomp::ptr pd = part_decomp_;
    config_parameters::ptr p = gtcparams_;

    SSTMAC_DEBUG << "gtc(" << p->mype_ << "):: starting field() \n";

    // finite difference for e-field in equilibrium unit
    double diffr = 0.5 / fa->deltar_;
    comp_->compute_loop(1, p->mpsi_, 1);

    double diffz = 0.5 / pd->deltaz_;
    //$omp parallel do private(i,j,k)
    comp_->compute_loop2(1, fa->mgrid_, 0, pd->mzeta_, 3);

    // d_phi/d_psi
    comp_->compute_loop2(1, fa->mgrid_, 0, pd->mzeta_, 3);

    // d_phi/d_theta
    //$omp parallel do private(i,k,j,ij,jt)
    comp_->compute_loop2(1, fa->mgrid_, 0, pd->mzeta_, 3);

    // send phi to right and receive from left
    comp_->compute_loop(1, fa->mgrid_, 2);

    int icount = fa->mgrid_;
    //!idest=mod(mype+1,numberpe)
    int idest = pd->right_pe_;
    int isource = pd->left_pe_;
    int isendtag = pd->myrank_toroidal_;
    int irecvtag = isource;
    MPI_Status istatus;

    MPI_SendRecv(NULL, icount, MPI_DOUBLE, idest, isendtag, NULL, icount,
        MPI_DOUBLE, isource, irecvtag, pd->toroidal_comm_, &istatus);

    //send phi to left and receive from right
    //!idest=mod(mype-1+numberpe,numberpe)
    idest = pd->left_pe_;
    //!isource=mod(mype+1,numberpe)
    isource = pd->right_pe_;
    //!isendtag=mype
    isendtag = pd->myrank_toroidal_;
    irecvtag = isource;

    MPI_SendRecv(NULL, icount, MPI_DOUBLE, idest, isendtag, NULL, icount,
        MPI_DOUBLE, isource, irecvtag, pd->toroidal_comm_, &istatus);

    // unpack phi_boundary and calculate E_zeta at boundaries, mzeta=1
    //$omp parallel do private(i,j,ii,jt,ij,pleft,pright)
    comp_->compute_loop(1, fa->mgrid_, 4);

    // adjust the difference between safety factor q and qtinv for fieldline coordinate
    //omp parallel do private(i,j,r,q,delq,ij)
    comp_->compute_loop2(1, fa->mgrid_, 0, pd->mzeta_, 1);

    // add (0,0) mode, d phi/d psi
    if (p->mode00_ == 1)
      {
        //omp parallel do private(i,j,k,ii,jt,r)
        comp_->compute_loop2(1, fa->mgrid_, 0, pd->mzeta_, 1);

      }

    // toroidal end point, pack end point (k=mzeta) data
    // send E to right and receive from left
    //$omp parallel do private(i)
    comp_->compute_loop(1, fa->mgrid_, 3);

    icount = 3 * fa->mgrid_;
    idest = pd->right_pe_;
    isource = pd->left_pe_;
    isendtag = pd->myrank_toroidal_;
    irecvtag = isource;

    MPI_SendRecv(NULL, icount, MPI_DOUBLE, idest, isendtag, NULL, icount,
        MPI_DOUBLE, isource, irecvtag, pd->toroidal_comm_, &istatus);

    // unpack end point data for k=0
    if (pd->myrank_toroidal_ == 0)
      {
        //$omp parallel do private(i,ii,jt)
        comp_->compute_loop(1, fa->mgrid_, 3);

      }
    else
      {
        //$omp parallel do private(i)
        comp_->compute_loop(1, fa->mgrid_, 3);

      }

    // poloidal end point
    //omp parallel do private(i,j)
    comp_->compute_loop2(1, p->mpsi_, 0, pd->mzeta_, 3);

    SSTMAC_DEBUG << "gtc(" << p->mype_ << "):: done with field() \n";
  }

  /**
   * ---------------------------------------------------------------------------------------
   * diagnosis
   * ---------------------------------------------------------------------------------------
   */
  void
  gtc_main::diagnosis()
  {

  }

  /**
   * ---------------------------------------------------------------------------------------
   * collision
   * ---------------------------------------------------------------------------------------
   */
  void
  gtc_main::collision(int istep, int neop, int neot, int neoz)
  {
    //short names
    field_array::ptr fa = field_array_;
    particle_decomp::ptr pd = part_decomp_;
    config_parameters::ptr p = gtcparams_;

    // ! read maxwell.dat
    if (istep == p->ndiag_)
      {
        // ! basic collision time, Braginskii
        // !     tauii=24.0-log(sqrt(0.46e14)/2555.0)
        // !     tauii=2.09e7*(2555.0)**1.5/(0.46e14*tauii*2.31)*sqrt(2.0)/utime
        //    !zeff=2.31

        // ! assume carbon impurity
        //  !!   tauii=23.0-log(sqrt(6.0*zeff*den_electron)/tem_ion**1.5)
        //  !!   tauii=2.09e7*(tem_ion)**1.5*sqrt(aspecie)/(den_electron*tauii*utime*zeff)

        MPI_Bcast(NULL, 100001, MPI_DOUBLE, 0, MPI_COMM_WORLD);

      }

    comp_->compute_loop(1, fa->mi_, 12);

    comp_->compute_loop(1, fa->mi_, 27);

    // ! global sum
    int icount = neop * neot * neoz;

    MPI_Allreduce(NULL, NULL, icount, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);

    MPI_Allreduce(NULL, NULL, icount, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);

    MPI_Allreduce(NULL, NULL, icount, MPI_DOUBLE, MPI_SUM, MPI_COMM_WORLD);

    // ! local conservation
    comp_->compute_loop(1, fa->mi_, 11);

    //  ! update velocity variables

  }

  /**
   * ---------------------------------------------------------------------------------------
   * poisson
   * @param something
   * ---------------------------------------------------------------------------------------
   */
  void
  gtc_main::poisson(int istep, int iflag)
  {
    //short names
    field_array::ptr fa = field_array_;
    particle_decomp::ptr pd = part_decomp_;
    config_parameters::ptr p = gtcparams_;

    int ipartd, nzeta, izeta1, izeta2;

    // ! number of gyro-ring
    int mring = 2;

    // ! number of summation: maximum is 32*mring+1
    int mindex = 32 * mring + 1;
    // !  mindex=53

    // ! gamma=0.75: max. resolution for k=0.577
    double gamma = 0.75;
    int iteration = 5;

    // initialize poisson solver
    if (istep == 1 && p->irk_ == 1 && iflag == 0)
      {
        comp_->compute_loop4(1, pd->mzeta_, 0, fa->mgrid_, 1, mring, 1, 8, 29);
        comp_->compute_loop2(1, pd->mzeta_, 0, fa->mgrid_, 3);
      }

    if (p->npartdom_ > 1 && (pd->mzeta_ % p->npartdom_) == 0)
      {
        //! mzeta is a multiple of npartdom so we can split the work of the
        //! k=1,mzeta loop between the processors having the same toroidal
        //! domain but a different particle domain.
        //! First we split mzeta into npartdom and determine the loop indices
        //! for each process part of the particle domain.
        ipartd = 1;
        nzeta = pd->mzeta_ / p->npartdom_;
        izeta1 = pd->myrank_partd_ * nzeta + 1;
        izeta2 = (pd->myrank_partd_ + 1) * nzeta;
      }
    else
      {
        ipartd = 0;
        izeta1 = 1;
        izeta2 = pd->mzeta_;
      }

    //   !!!do k=1,mzeta
    comp_->compute_loop2(izeta1, izeta2, 1, fa->mgrid_, 1);
    comp_->compute_loop2(2, iteration, 1, fa->mgrid_, 4);

    if (ipartd == 1)
      {
        //   ! Since the work was split between the processors of different particle
        //   ! domains, we need to gather the full array phi on these processors.

        comp_->compute_loop2(izeta1, izeta2, 1, fa->mgrid_, 1);

        MPI_Allgather(NULL, nzeta * fa->mgrid_, MPI_DOUBLE, NULL, nzeta * fa->mgrid_,
            MPI_DOUBLE, pd->partd_comm_);

        comp_->compute_loop2(1, pd->mzeta_, 1, fa->mgrid_, 1);

      }

    // ! in equilibrium unit
    //  !$omp parallel do private(i)
    comp_->compute_loop2(0, p->mpsi_, 1, pd->mzeta_, 2);

  }

}
