/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include "gtc_main.h"

namespace gtc
{

  using namespace std;
  using namespace sstmac;
  using namespace sstmac::sw;

  /**
   * ---------------------------------------------------------------------------------------
   * shifti
   * ---------------------------------------------------------------------------------------
   */
  void
  gtc_main::shifti()
  {
    //short names
    field_array::ptr fa = field_array_;
    particle_decomp::ptr pd = part_decomp_;
    config_parameters::ptr p = gtcparams_;

    int nzion = 2 * p->nparam_;

    int m0 = 1;
    int iteration = 0;

    for (int iteration = 0; iteration < p->numberpe_; iteration++)
      {

        if (m0 <= fa->mi_)
          {
            // !$omp parallel do private(m)
            comp_->compute_loop(1, fa->mi_, 1);

            //! NOTE: The vector kzelectron is also used in chargee and pushe to temporarily
            // !    store the closest zeta grid point to the electron "m". S.Ethier 10/7/04

            //  !  This section of code replaces the section above when the compilation does
            //   !  NOT include the OpenMP support option. Temporary arrays msleft and msright
            //   !  are not needed as well as the extra code for thread work distribution.
            comp_->compute_loop(m0, fa->mi_, 9);

          }

        if (iteration > 1)
          {
            //! total # of particles to be shifted

            MPI_Allreduce(NULL, NULL, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
          }

        //! an extra space to prevent zero size when msendright(1)=msendleft(1)=0

        //  ! pack particle to move right
        //  !$omp parallel do private(m)
        comp_->compute_loop(1, 10, 2);

        //  ! pack particle to move left
        //  !$omp parallel do private(m)
        comp_->compute_loop(1, 10, 2);

        //  ! # of particles remain on local PE

        //   ! fill the hole
        comp_->compute_loop(1, 20, 8);

        //  ! send # of particle to move right

        int idest = pd->right_pe_;
        int isource = pd->left_pe_;
        int isendtag = pd->myrank_toroidal_;
        int irecvtag = isource;
        MPI_Status istatus;
        MPI_SendRecv(NULL, 2, MPI_INT, idest, isendtag, NULL, 2, MPI_INT,
            isource, irecvtag, pd->toroidal_comm_, &istatus);

        // ! send particle to right and receive from left

        int isendcount = 5 * nzion;
        int irecvcount = 5 * nzion;

        MPI_SendRecv(NULL, isendcount, MPI_DOUBLE, idest, isendtag, NULL,
            irecvcount, MPI_DOUBLE, isource, irecvtag, pd->toroidal_comm_,
            &istatus);

        // ! send # of particle to move left

        idest = pd->left_pe_;
        isource = pd->right_pe_;
        isendtag = pd->myrank_toroidal_;
        irecvtag = isource;
        MPI_SendRecv(NULL, 2, MPI_INT, idest, isendtag, NULL, 2, MPI_INT,
            isource, irecvtag, pd->toroidal_comm_, &istatus);

        //  ! send particle to left and receive from right

        isendcount = 5 * nzion;
        irecvcount = 5 * nzion;

        MPI_SendRecv(NULL, isendcount, MPI_DOUBLE, idest, isendtag, NULL,
            irecvcount, MPI_DOUBLE, isource, irecvtag, pd->toroidal_comm_,
            &istatus);

        //  ! unpack particle, particle moved from left
        //  !$omp parallel do private(m)
        comp_->compute_loop(1, 10, 2);

        // ! particle moved from right
        //  !$omp parallel do private(m)
        comp_->compute_loop(1, 10, 2);

      }

  }

  /**
   * ---------------------------------------------------------------------------------------
   * shifte
   * ---------------------------------------------------------------------------------------
   */
  void
  gtc_main::shifte()
  {
    //short names
    field_array::ptr fa = field_array_;
    particle_decomp::ptr pd = part_decomp_;
    config_parameters::ptr p = gtcparams_;

    int nzelectron = 12;

    int m0 = 1;
    int iteration = 0;

    for (int iteration = 0; iteration < p->numberpe_; iteration++)
      {

        if (m0 <= fa->me_)
          {
            // !$omp parallel do private(m)
            comp_->compute_loop(1, fa->me_, 1);

            //! NOTE: The vector kzelectron is also used in chargee and pushe to temporarily
            // !    store the closest zeta grid point to the electron "m". S.Ethier 10/7/04

            //  !  This section of code replaces the section above when the compilation does
            //   !  NOT include the OpenMP support option. Temporary arrays msleft and msright
            //   !  are not needed as well as the extra code for thread work distribution.
            comp_->compute_loop(m0, fa->me_, 9);

          }

        if (iteration > 1)
          {
            //! total # of particles to be shifted

            MPI_Allreduce(NULL, NULL, 1, MPI_INT, MPI_SUM, MPI_COMM_WORLD);
          }

        //! an extra space to prevent zero size when msendright(1)=msendleft(1)=0

        //  ! pack particle to move right
        //  !$omp parallel do private(m)
        comp_->compute_loop(1, 10, 2);

        //  ! pack particle to move left
        //  !$omp parallel do private(m)
        comp_->compute_loop(1, 10, 2);

        //  ! # of particles remain on local PE

        //   ! fill the hole
        comp_->compute_loop(1, 20, 8);

        //  ! send # of particle to move right

        int idest = pd->right_pe_;
        int isource = pd->left_pe_;
        int isendtag = pd->myrank_toroidal_;
        int irecvtag = isource;
        MPI_Status istatus;
        MPI_SendRecv(NULL, 2, MPI_INT, idest, isendtag, NULL, 2, MPI_INT,
            isource, irecvtag, pd->toroidal_comm_, &istatus);

        // ! send particle to right and receive from left

        int isendcount = 5 * nzelectron;
        int irecvcount = 5 * nzelectron;

        MPI_SendRecv(NULL, isendcount, MPI_DOUBLE, idest, isendtag, NULL,
            irecvcount, MPI_DOUBLE, isource, irecvtag, pd->toroidal_comm_,
            &istatus);

        // ! send # of particle to move left

        idest = pd->left_pe_;
        isource = pd->right_pe_;
        isendtag = pd->myrank_toroidal_;
        irecvtag = isource;
        MPI_SendRecv(NULL, 2, MPI_INT, idest, isendtag, NULL, 2, MPI_INT,
            isource, irecvtag, pd->toroidal_comm_, &istatus);

        //  ! send particle to left and receive from right

        isendcount = 5 * nzelectron;
        irecvcount = 5 * nzelectron;

        MPI_SendRecv(NULL, isendcount, MPI_DOUBLE, idest, isendtag, NULL,
            irecvcount, MPI_DOUBLE, isource, irecvtag, pd->toroidal_comm_,
            &istatus);

        //  ! unpack particle, particle moved from left
        //  !$omp parallel do private(m)
        comp_->compute_loop(1, 10, 2);

        // ! particle moved from right
        //  !$omp parallel do private(m)
        comp_->compute_loop(1, 10, 2);

      }

  }

}
