/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_ATOM_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_MINI_MINIMD_MINIMD_ATOM_H_INCLUDED

#include <minimd.h>
#include <vector>

namespace mini {

  // Ignored types.
  class lib_compute_minimd;
  class sstmac::sw::mpiapi;

  /**
   * Take over the responsibilities of class Atom in miniMD.
   */
  class minimd::atom : public ptr_type {
  public:

    virtual std::string to_string() const{
            return "atom";
          }

    typedef boost::intrusive_ptr<atom> ptr;
    struct Box {
      double xprd, yprd, zprd;
      double xlo, xhi;
      double ylo, yhi;
      double zlo, zhi;
      void set_prd(double x, double y, double z) {
        xprd = x; yprd = y; zprd = z;
      }
    };

    int natoms;
    std::vector<int> neighatoms; // Added to keep track of everybodies nlocal.
    int nlocal;
    int nmax;
    Box box;

    void init(const lib_compute_minimd::ptr&, const mpiapi::ptr&) {
      // all atom:: operations are rolled into runtime overhead for other calls.
    }
  };

} 

#endif
