/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <fibonacci/hpx_fibonacci.h>

#include <hpx/lcos/eager_future.h>
#include <sstmac/sstmacro.h>

#include <hpx/runtime/actions/hpx_action.h>



using namespace sstmac;
using namespace sstmac::sw;
using namespace std;

char FIB_NAME[] = "fib";

namespace hpx
{
  namespace fibonacci
  {

    SSTRegisterApp("hpx_fibonacci", hpx_fibonacci);

    using namespace std;

    boost::uint64_t
    fibonacci(boost::uint64_t n);

    typedef hpx::lcos::eager_future<boost::uint64_t> fibonacci_future;

    boost::uint64_t
    fibonacci(boost::uint64_t n)
    {
      if (n < 2)
        return n;

      HPX_DBG
          << "hpx_fibonacci::fib::fibonacci -- starting a recurse with value "
          << n << "\n";

      // We restrict ourselves to execute the Fibonacci function locally.
      nodeaddress::ptr loc = hpx::actions::find_here();

      // Invoking the Fibonacci algorithm twice is inefficient.
      // However, we intentionally demonstrate it this way to create some
      // heavy workload.
      hpx::hpx_action_params1<boost::uint64_t>::ptr p1 =
          hpx::hpx_action_params1<boost::uint64_t>::construct(n - 1);
      hpx::hpx_action_params1<boost::uint64_t>::ptr p2 =
          hpx::hpx_action_params1<boost::uint64_t>::construct(n - 2);
      fibonacci_future::ptr n1 = fibonacci_future::construct(FIB_NAME, loc, p1);
      fibonacci_future::ptr n2 = fibonacci_future::construct(FIB_NAME, loc, p2);

      hpx::actions::compute(timestamp(10e-9));

      return n1->get() + n2->get(); // wait for the Futures to return their values
    }

    void
    hpx_fibonacci::consume_params(const sim_parameters::ptr &params)
    {
      n_value_ = params->get_optional_long_param("hpx_fibonacci_n_value", 10);

    }

    void
    hpx_fibonacci::register_actions()
    {
      // Any global function needs to be wrapped into a plain_action if it should be
      // invoked as a HPX-thread.
      typedef hpx::actions::plain_result_action1<boost::uint64_t, // result type
          boost::uint64_t, // argument
          fibonacci // function
      > fibonacci_action;
      hpx_register_action<fibonacci_action>(FIB_NAME); //this creates the library, which can respond to requests of it
    }

    void
    hpx_fibonacci::register_components()
    {

    }

    void
    hpx_fibonacci::hpx_main()
    {
      SSTMAC_DEBUG << "Hello World, i'm a parallex fibonacci app. \n";

      hpx::hpx_action_params1<boost::uint64_t>::ptr p1 =
          hpx::hpx_action_params1<boost::uint64_t>::construct(n_value_);

      nodeaddress::ptr loc = os_->get_my_addr();
      fibonacci_future::ptr n1 = fibonacci_future::construct(FIB_NAME, loc, p1);

      boost::uint64_t result = n1->get();
      SSTMAC_DEBUG << "I Figured it out! It's : " << result << "\n";
    }

} }
