/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <rma/hpx_rma_element.h>
//#include <rma/hpx_rma_element_actions.h>
#include <hpx/runtime/hpx_thread_manager.h>

using namespace sstmac;
using namespace sstmac::sw;

namespace hpx
{
  namespace rma 
  {

  char INIT_NAME[] = "hpx_rma_element::init";
  char ADD_NAME[] = "hpx_rma_element::add";
  char PRINT_NAME[] = "hpx_rma_element::print";

  int
  hpx_random_mem_access::hpx_rma_element::init(std::size_t s)
  {
    mtx_->lock();

    arg_ = s;
    arg_init_ = s;

    mtx_->unlock();
    return 0;
  }

  int
  hpx_random_mem_access::hpx_rma_element::add()
  {
    mtx_->lock();

    arg_ += 1;

    mtx_->unlock();
    return 0;
  }

  int
  hpx_random_mem_access::hpx_rma_element::print()
  {
    mtx_->lock();

    SSTMAC_DEBUG << "Element " << arg_init_ << " with gid " << gid_.gid_
          << " has value " << arg_ << "\n";

    mtx_->unlock();
    return 0;
  }

  void
  hpx_random_mem_access::hpx_rma_element::register_actions(
      boost::intrusive_ptr<hpx::hpx_thread_manager> threadman)
  {
    typedef hpx::actions::component_result_action1<int,
        hpx_random_mem_access::hpx_rma_element, std::size_t, // argument
        &hpx_random_mem_access::hpx_rma_element::init // function
    > init_action;

    typedef hpx::actions::component_result_action0<int,
        hpx_random_mem_access::hpx_rma_element,
        &hpx_random_mem_access::hpx_rma_element::add // function
    > add_action;

    typedef hpx::actions::component_result_action0< int,
        hpx_random_mem_access::hpx_rma_element,
        &hpx_random_mem_access::hpx_rma_element::print // function
    > print_action;

    threadman->register_action<init_action>(INIT_NAME,
        init_action::construct());
    threadman->register_action<add_action>(ADD_NAME, add_action::construct());
    threadman->register_action<print_action>(PRINT_NAME,
        print_action::construct());
  }

} }
