/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SKELETONS_PARALLEX_HPX_RMA_ELEMENT_H_INCLUDED
#define SSTMAC_SOFTWARE_SKELETONS_PARALLEX_HPX_RMA_ELEMENT_H_INCLUDED

#include <rma/hpx_random_mem_access.h>
#include <hpx/components/hpx_component.h>
#include <hpx/lcos/local_mutex.h>

namespace hpx
{
  namespace rma
  {


    class hpx_random_mem_access::hpx_rma_element : public hpx::components::hpx_component
    {
    protected:
      class rma_stubs;

      std::size_t arg_;
      std::size_t arg_init_;

      hpx::lcos::local_mutex::ptr mtx_;

      // ---- these are the functions that actually do something,
      // ------- they are protected so that only actions can access them
      int
      init(std::size_t s);

      int
      add();

      int
      print();

      // -------------------------------- //
      hpx_rma_element(hpx::globalid gid) :
          hpx_component(gid)
      {
        mtx_ = hpx::lcos::local_mutex::construct();
      }

    public:
      typedef boost::intrusive_ptr<hpx_rma_element> ptr;
      typedef boost::intrusive_ptr<rma_stubs> stub_ptr;
      typedef rma_stubs stubs;

      static ptr
      construct(hpx::globalid gid)
      {
        return ptr(new hpx_rma_element(gid));
      }

      virtual
      ~hpx_rma_element()
      {
      }

      static std::string
      component_name()
      {
        return "hpx_random_mem_access::hpx_rma_element";
      }

      virtual void
      register_actions(boost::intrusive_ptr<hpx::hpx_thread_manager> threadman);

      virtual hpx::components::hpx_component::ptr
      clone(hpx::globalid gid)
      {
        return construct(gid);
      }

    };

  } 
}
#endif /* HPX_RMA_ELEMENT_H_ */
