/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <networkmodels/packet/packetrackswitch.h>

#include <network_m.h>

namespace sstmac { namespace omnetpp {




  stat_time_window_avg::ptr packetrackswitch::stat_Q_wait_ = NULL;
  stat_time_window_sum::ptr packetrackswitch::stat_bytes_ = NULL;
  stat_time_window_sum_duration::ptr packetrackswitch::stat_pwr_ = NULL;

  Define_Module(packetrackswitch);

  using namespace std;



  void
  packetrackswitch::initialize()
  {

    //latencyPerHop = getParam("latencyPerHop");

    if(!stat_Q_wait_)
      stat_Q_wait_ = stat_time_window_avg::construct("<stats> network",
          "simulations/results/stats_packet_Qwait.csv");

    if(!stat_bytes_)
      stat_bytes_ =  stat_time_window_sum::construct("<stats> network",
          "simulations/results/stats_packet_bytes.csv");

    if(!stat_pwr_)
      stat_pwr_ =  stat_time_window_sum_duration::construct("<stats> network",
          "simulations/results/stats_power_network.csv");

    int myrackid = par("rackid");
    int npr = par("rack_nodes");
    int nrc = par("numRackConnections");

    omnetpp_address::ptr me = omnetpp_address::construct(-1, -1, -1, myrackid);

    int topP1 = par("topParam1");
    int topP2 = par("topParam2");
    int topP3 = par("topParam3");

    string top = par("networkTopology");

    init(me, npr, nrc, top, topP1, topP2, topP3);

    displayConnections = par("updateConnectionDisplays");

    for (int i = 0; i < numRackConnections; i++)
      {
        inports[i] = gate("ports$i", i);
        outports[i] = gate("ports$o", i);

        //if (outports[i]->findTransmissionChannel() != NULL
        // )
        //  register_channel(outports[i]->getTransmissionChannel(), this);

        outQs[i] = new cPacketQueue();

        cMessage* cm = new cMessage("transFinish");
        cm->setKind(i);
        transFinish[cm] = (long) cm;
        transMsg[i] = cm;

        //bytes_[i] = 0;
      }

    updateInterval_ = 1;

    //if (displayConnections){
    //  displayUpdate = new cMessage("displayUpdate");
    //   scheduleAt(updateInterval_, displayUpdate);
    // }

  }

  void
  packetrackswitch::finish()
  {
    for (int i = 0; i < numRackConnections; i++)
      {
        delete outQs[i];
        delete transMsg[i];

      }

    // if (displayConnections)
    //   delete displayUpdate;

  }

  void
  packetrackswitch::handleMessage(cMessage* cmsg)
  {

    /*if (cmsg == displayUpdate)
 {

 for (int i = 0; i < numRackConnections; i++)
 {
 if (outports[i]->findTransmissionChannel() != NULL)
 {
 cDatarateChannel* ch = check_and_cast<cDatarateChannel*>(
 outports[i]->getTransmissionChannel());
 double val = ((double)bytes_[i] * 8.0 / updateInterval_.dbl()) / (double)ch->getDatarate();
 update_channel_display(outports[i]->getTransmissionChannel(), this,
 val);
 bytes_[i] = 0;
 }
 }

 scheduleAt(simTime() + updateInterval_, displayUpdate);
 }
 else*/if (transFinish[cmsg] == (long) cmsg)
   {
     tryOutQ(cmsg->getKind());
   }
 else
   {

     NetworkMsg* n = check_and_cast<NetworkMsg*>(cmsg);

     SSTMAC_DEBUG << "router " << myAddr->to_string() << ": received packet " << (long)n << "\n";
     omnetpp_address::ptr addr = n->getToAddr();

     bool isDimChange = false;
     int outport = router_->route(n, addr, 0, isDimChange);

     outQs[outport]->insert(n);
     entrytimes_[cmsg] = simTime();
     tryOutQ(outport);

   }

  }

  void
  packetrackswitch::tryOutQ(int port)
  {

    if (!outQs[port]->empty())
      {
        simtime_t txFinishTime =
            outports[port]->getTransmissionChannel()->getTransmissionFinishTime();
        if (txFinishTime <= simTime())
          { //channel free
            //std::cout << "NIC: sending message " << endl;
            cPacket* msg = outQs[port]->pop();
            send(msg, outports[port]);

            timestamp start = timestamp(simTime().dbl());

            double wait = (simTime() - entrytimes_[(cMessage*) msg]).dbl();
            stat_Q_wait_->record(start, wait);
            entrytimes_.erase((cMessage*) msg);

            stat_bytes_->record(start, msg->getByteLength());


            double pwr = 0.8;
            timestamp duration = timestamp((outports[port]->getTransmissionChannel()->getTransmissionFinishTime() - simTime()).dbl());
            //std::cout << "packetswitch - recording power: start=" << start << ", duration=" << duration << std::endl;
            stat_pwr_->record(start, 1.5, duration);

            SSTMAC_DEBUG << "rackswitch " << myAddr->to_string()
                << ": forwarding packet of size " << (int) msg->getByteLength()
                << " to port " << port << "\n";

            //bytes_[port] += msg->getByteLength();

            tryOutQ(port);
          }
        else if (!transMsg[port]->isScheduled())
          {
            scheduleAt(txFinishTime, transMsg[port]);
          }
      }

  }

} }
