/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_OMNETPP_NETWORKMODELS_ROUTING_3DCUBEROUTER_H_INCLUDED
#define SSTMAC_BACKENDS_OMNETPP_NETWORKMODELS_ROUTING_3DCUBEROUTER_H_INCLUDED

#include <networkmodels/routing/router.h>

namespace sstmac { namespace omnetpp {


  class cube3drouter: public router {
  public:
    cube3drouter(omnetpp_address::ptr m, int npr, int x, int y, int z) :
      router(m, npr), numX(x), numY(y), numZ(z) {
      myX = me->rack() % numX;
      myY = (me->rack() / numX) % numY;
      myZ = me->rack() / (numX * numY);

      SSTMAC_DEBUG << "router rack id = " << me->rack() << "; coord = (" << myX << ", " << myY << ", " << myZ << ")\n";
    }

    virtual int route(NetworkMsg* msg, omnetpp_address::ptr to, int inport, bool &dim_change);

  protected:
    int numX;
    int numY;
    int numZ;

    int myX;
    int myY;
    int myZ;
  };

} }
#endif
