/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <networkmodels/routing/spidergonrouter.h>
#include <math.h>
namespace sstmac
{
  namespace omnetpp
  {

    int
    spidergonrouter::route(NetworkMsg* msg, omnetpp_address::ptr to, int inport, bool &dim_change)
    {

      int mine = me->rack();
      int him = to->rack();


      SSTMAC_DEBUG << "Router(" << mine << "): Routing to address " << to->to_string() << ": (" << him<< ")";

      int ret = -1;

      int diff = std::min(abs(him - mine), std::min(numX*numY + him - mine, numX*numY + mine - him));

      dim_change = (inport <= nodesPerRack);

      if(mine == him){
    	  dim_change = false;
    	  ret = to->node();
      }else if(diff > numX*numY / 4){
    	  ret = nodesPerRack + 2;
      }else{
    	  dim_change = dim_change || (inport == nodesPerRack + 2);
    	  if(him > mine || (numX*numY + him - mine < mine - him))
    		  ret = nodesPerRack + 0;
    	  else
    		  ret = nodesPerRack + 1;
      }

      SSTMAC_DEBUG << ": outport " << ret << "\n";

      return ret;
    }

  }
}
