/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/backends/omnetpp/node/nic/mpicollsecnic_gate.h>

#include <network_m.h>
//#include "mpi_m.h"

namespace sstmac {
namespace omnetpp {

Define_Module(mpicollsecnic_gate);

void mpicollsecnic_gate::initialize() {
	fromProc = gate("toProcessor$i", 0);
	toProc = gate("toProcessor$o", 0);
	fromNet = gate("toNetwork$i", 0);
	toNet = gate("toNetwork$o", 0);
	toCollNet = gate("toNetwork$o", 1);
	fromCollNet = gate("toNetwork$i", 1);

}

void mpicollsecnic_gate::finish() {

}

void mpicollsecnic_gate::handleMessage(cMessage *cmsg) {
	cGate* arrival = cmsg->getArrivalGate();

	if (arrival == fromProc) {
		CoreMsg* c = check_and_cast<CoreMsg*>(cmsg);

		if (c->getMsgType() == APP) {

		} else {
			opp_error("mpicollsecnic_gate: not expecting a non-APP msg");
		}

	} else if (arrival == fromNet || arrival == fromCollNet) {

		send(cmsg, toProc);

	}
}

}
}
