/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
/*
#include "Core.h"

#include "Node/Skeletons.h"

#include "SSTSF/AMI/environment.h"
#include "SSTSF/Process/operatingsystem.h"
#include "SSTSF/AMI/hardware_descriptor.h"

#include "SSTSF/Messages/sst_message.h"

#include "rackallocator.h"

#include "randomindexer.h"
#include "fixedindexer.h"
#include "fromfileindexer.h"

#include "rackspace.h"

//map<int, Indexer*> Core::appIndexers;
map<sstmac::sw::appid, sstmac::sw::environment::ptr > Core::env_;

Define_Module(Core)
;

Core::Core() :
  dbg(debug_sstmedium::DBG_CORE)
{


}

Core::~Core()
{

}

void
Core::initialize()
{

  int c = par("coreid");

  int p = par("procid");

  int n = par("nodeid");

  int r = par("rackid");

  myid = Address(c, p, n, r);

  //app = NULL;

  //dbg.type = debug::DBG_CORE;

  Skeletons::init_skeletons();

  coresPerProc = par("coresPerProc");
  numOfNodes = par("numOfNodes");
  procsPerNode = par("procsPerNode");
  nodesPerRack = par("nodesPerRack");
  numRacks = par("numRacks");

  outgate = gate("toNoc$o");
  ingate = gate("toNoc$i");

  // ------- operating system -------
  boost::shared_ptr<sstmac::sw::hardware_descriptor> hw =
      sstmac::sw::hardware_descriptor::construct(coresPerProc);
  opsys = sstmac::sw::operatingsystem::construct(hw);

  opsys->setAMI(sstmac::sw::AMI::AMI_COMM, this);
  opsys->setAMI(sstmac::sw::AMI::AMI_COMPUTE, this);

  // ---- get parameters and such

  int numOfApps = par("numOfApps");
  nextApp = 0;

  for (int i = 0; i < numOfApps; i++)
    {
      stringstream ss;

      ss << "app" << i << "Name";
      string temp1 = par(ss.str().c_str());
      appNames[i] = temp1;
      ss.str("");

      if (i == 0)
        appStartTimes[i] = 0;
      else
        {
          ss << "app" << i << "StartTime";
          appStartTimes[1] = par(ss.str().c_str());
          ss.str("");
        }

      ss << "app" << i << "Size";
      appSizes[i] = par(ss.str().c_str());
      ss.str("");

      appIndexers[i] = NULL;

      appStringParams[i] = new list<string> ();

      ss << "app" << i << "NumParams";
      int num_params = par(ss.str().c_str());
      ss.str("");

      for (int j = 0; j < num_params; j++)
        {

          ss << "app" << i << "Param" << j;
          string temp2 = par(ss.str().c_str());
          appStringParams[i]->push_back(temp2);
          ss.str("");
        }

    }

  appStartTimes[numOfApps] = -1;

  //----- self messages
  //computeMsg = new cMessage("computeMsg");
  for (int i = 0; i < coresPerProc; i++)
    {
      computeMessages[new cMessage("computemessage")]
          = sstmac::sw::sst_message::null();
    }

  appStartMsg = new cMessage("appStartMsg");

  // ---- set up master core
  if (is_master_core())
    {
      //only the master makes new environments.
      for (int i = 0; i < numOfApps; i++)
        {
          sstmac::sw::appid aid(i);
          env_[aid] = sstmac::sw::environment::construct();
        }

      newAppMsg = new cMessage("newAppMsg");

      string allocName = par("allocatorName");

      instantiate_allocator(allocName);

      //appspace = fullspace;

      scheduleAt(0, newAppMsg);

    }

  //everyone should set the environment pointers
  for (int i = 0; i < numOfApps; i++)
    {
      sstmac::sw::appid aid(i);

      opsys->set_environment(env_[aid], aid);
    }
}

void
Core::finish()
{

  opsys->apps_kill();

  map<int, list<string>*>::iterator it;
  for (it = appStringParams.begin(); it != appStringParams.end(); it++)
    delete it->second;

  map<cMessage*, boost::shared_ptr<sstmac::sw::sst_message> >::iterator it2;
  for (it2 = computeMessages.begin(); it2 != computeMessages.end(); it2++)
    delete it2->first;

  //delete computeMsg;
  delete appStartMsg;

  if (is_master_core())
    delete newAppMsg;

}

void
Core::instantiate_allocator(string allocName)
{

  if (allocName.compare("rack") == 0)
    {

      allocator = new rackallocator(numOfNodes, coresPerProc, procsPerNode,
          nodesPerRack, numRacks);

    }
  else if (allocName.compare("rackblock") == 0)
    {

    }
  else if (allocName.compare("system") == 0)
    { //the default whole-system allocator
      allocator = new resourceallocator(numOfNodes, coresPerProc, procsPerNode,
          nodesPerRack, numRacks);
    }
  else
    {
      std::cout << "Allocator Name: " << allocName << endl;
      opp_error("Core: unable to find allocator.");
    }

}

bool
Core::is_master_core()
{
  //return ((myid.rackid == 0 || myid.rackid < 0) && myid.nodeid == 0 && myid.id
  //    == 0 && myid.procid == 0);
}

void
Core::instantiate_indexer(Space* appspace, int appid)
{

  if (appIndexers[appid] == NULL)
    {
      dbg << "Core " << myid.to_string() << ": instantiating app indexer \n";

      string indexerName = par("appIndexerName");

      if (indexerName.compare("random") == 0)
        {
          appIndexers[appid] = new randomindexer(appspace);
        }
      else if (indexerName.compare("fixed") == 0)
        {
          appIndexers[appid] = new fixedindexer(appspace);
        }
      else if (indexerName.compare("fromFile") == 0)
        {
          string ipar = par("appIndexerParam1");
          appIndexers[appid] = new fromfileindexer(appspace, ipar);
        }
    }

}

void
Core::instantiate_app(Indexer* appind, std::string appname,
    sstmac::sw::softwareid newid)
{

  dbg << "Core " << myid.to_string() << ": instantiating app" << "\n";

  if (Skeletons::apps.find(appname) == Skeletons::apps.end())
    {

      opp_error("Application %s is not in the app factory.", appname.c_str());
    }

  sstmac::sw::app::ptr theapp = Skeletons::apps[appname]->clone(newid);
  // boost::shared_ptr<sstmac::sw::parameter_configurator> params =
  //   theapp->get_param_config();

  //consume parameters
  //list<string>::iterator it;
  // for (it = appStringParams[newid.app_.id]->begin(); it
  //    != appStringParams[newid.app_.id]->end(); it++)
  //  {
  //    params->consume_parameter(*it);
  //  }

  opsys->add_application(theapp, *appStringParams[newid.app_.id]);
}

Space*
Core::instantiate_space(string spacename, int spaceparam1, int spaceparam2)
{
  Space* ret = NULL;

  dbg << "Core " << myid.to_string() << ": instantiating " << spacename
      << " space with params " << spaceparam1 << " and " << spaceparam2 << "\n";

  if (spacename.compare("system") == 0)
    {
      ret = new systemspace(numOfNodes, coresPerProc, procsPerNode,
          nodesPerRack, numRacks);
    }
  else if (spacename.compare("rack") == 0)
    {

      ret = new rackspace(spaceparam1, spaceparam2, numOfNodes, coresPerProc,
          procsPerNode, nodesPerRack, numRacks);
    }
  else
    {
      std::cout << "Space name: " << spacename << endl;
      opp_error("Core: don't know how to instantiate space");

    }

  return ret;
}

bool
Core::launch_app(Indexer* index, Space* space, std::string appname, int aid,
    int appsize)
{

  dbg << "Core: master: launching " << appsize << " app instances" << "\n";

  bool setmyown = false;

  AppLaunchMsg* launch = new AppLaunchMsg();
  launch->setMsgType(OS_LAUNCH);
  launch->setByteLength(APP_SIZE + OS_MSG_HEADER_SIZE);

  launch->setSpaceName(space->getName().c_str());
  launch->setSpaceParam1(space->get_space_param1());
  launch->setSpaceParam2(space->get_space_param2());

  launch->setAppname(appname.c_str());
  launch->setAppID(aid);
  launch->setAppsize(appsize);

  //int ranks = space->get_num_slots();


  for (int i = 0; i < appsize; i++)
    {
      boost::shared_ptr<Address> n = index->getLocation(i);
      sstmac::sw::appid a(aid);
      sstmac::sw::taskid t(i);
      sstmac::sw::softwareid sid(a, t);
    //  env_[a]->add_item(t, n); // ------ this is MAGIC, to save on space --------- //
      dbg << "Adding to environment map - sid: " << sid.to_string()
          << ", addr: " << n->to_string() << "\n";
      if (!n->equals(myid))
        {
          AppLaunchMsg* msg = launch->dup();
          msg->setInstanceID(i);

          msg->setToAddr(*n);
          msg->setFromAddr(myid);

          sendDelayed(msg, SEND_DELAY, outgate);

        }
      else
        {
          instantiate_app(index, appname, sid);

          dbg << "Core " << myid.to_string() << ": launching app id " << aid
              << ", instance " << i << "\n";
          scheduleAt(simTime() + APP_START_DELAY, appStartMsg);

          setmyown = true;
        }

    }

  delete launch;
  //appslaunched = ranks;

  return setmyown;
}

void
Core::handleMessage(cMessage* cmsg)
{
  //std::cout << "Core " << myid << ": handle message" << endl;
  if (computeMessages.find(cmsg) != computeMessages.end())
    {
      dbg << "Core(" << myid.to_string() << "): done computing "
                    << "\n";

      boost::shared_ptr<sstmac::sw::sst_message> unblock = computeMessages[cmsg];

      if (!suspendedQ.empty())
        {
          dbg << "Core(" << myid.to_string() << "): executing suspended kernel "
                              << "\n";

          executeKernel(sstmac::sw::AMI::AMI_COMP_TIME, suspendedQ.front());
          suspendedQ.pop();
        }

      //opsys->unblock(unblock);
      opsys->handle_message(unblock);
      //app->doneComputing();
    }
  else if (cmsg == appStartMsg)
    {
      opsys->start_apps();
    }
  else if (cmsg == newAppMsg)
    {
      dbg << "Master Core: starting app " << nextApp << "\n";
      //get a subset of the space here with a resource allocator
      Space *appspace = allocator->allocate(appSizes[nextApp]);

      if (appspace->get_num_slots() < appSizes[nextApp])
        {
          std::cout
              << "--- WARNING: allocator returned a space with less nodes than was requested"
              << endl;
          std::cout << "Requested: " << appSizes[nextApp] << endl;
          std::cout << "Allocated: " << appspace->get_num_slots() << endl;
          std::cout << "--- Resizing the app to fit in the space" << endl;

          appSizes[nextApp] = appspace->get_num_slots();
        }

      instantiate_indexer(appspace, nextApp);

      //Launch on given hardware space
      if (!launch_app(appIndexers[nextApp], appspace, appNames[nextApp],
          nextApp, appSizes[nextApp]))
        delete appspace;

      delete appIndexers[nextApp]; // get rid of it, so we're not keeping the mapping around twice (already in env_)

      nextApp++;

      if (appStartTimes[nextApp] > 0)
        {
          scheduleAt(appStartTimes[nextApp], newAppMsg);
        }

    }
  else if (cmsg->getArrivalGate() == ingate)
    { //message arriving in from the NoC
      CoreMsg *m = check_and_cast<CoreMsg*> (cmsg);

      if (!m->getToAddr().equals(myid))
        {
          std::cout << "My ID: " << myid.to_string() << endl;
          std::cout << "Msg Dest: " << m->getToAddr().to_string() << endl;
          opp_error("Application: got a message destined for NOT me");
        }

      switch (m->getMsgType())
        {
      case OS_LAUNCH:
        {

          AppLaunchMsg* launch = check_and_cast<AppLaunchMsg*> (m);

          //std::cout << "Core " << myid << ": received an OS_LAUNCH" << endl;
          //first, instantiate the space that describes where
          // this app lives in the physical system
          std::string spacename = launch->getSpaceName();

          Space* thespace = instantiate_space(spacename,
              launch->get_space_param1(), launch->get_space_param2());

          instantiate_indexer(thespace, launch->getAppID());

          sstmac::sw::appid a(launch->getAppID());
          sstmac::sw::taskid t(launch->getInstanceID());
          sstmac::sw::softwareid sid(a, t);
          //now intantiate and start
          instantiate_app(appIndexers[launch->getAppID()], launch->getAppname(),
              sid);

          dbg << "Core " << myid.to_string() << ": launching app id "
              << launch->getAppID() << ", instance " << launch->getInstanceID()
              << "\n";
          if (!appStartMsg->isScheduled())
            {
              scheduleAt(simTime() + APP_START_DELAY, appStartMsg);
            }
          break;
        }

      case APP:
        {

          dbg << "Core " << myid.to_string() << ": app message arrived \n ";
          ApplicationMsg* appmsg = check_and_cast<ApplicationMsg*> (m);
          payloadpod ppod = appmsg->getMsgcontent();
          boost::shared_ptr<sstmac::sw::sst_message> bsstm = ppod.load;
          // sst_message sstmsg = appmsg->getMsgcontent();


          opsys->handle_message(bsstm);

          break;
        }

      case NIC_SEND_ACK:
        {

          dbg << "Core " << myid.to_string() << ": NIC send ack arrived ";
          ApplicationMsg* appmsg = check_and_cast<ApplicationMsg*> (
              m->decapsulate());

          payloadpod ppod = appmsg->getMsgcontent();
          boost::shared_ptr<sstmac::sw::sst_message> bsstm = ppod.load;

          dbg << bsstm->to_string() << "\n";
          opsys->handle_message(bsstm);

          //  ((CommunicationAPI*) app->getAPI(appmsg->getApiID()))->nicSendAck(
          //     appmsg);
          //((CommunicationAPI*)apis[appmsg->getApiID()])->nicSendAck(appmsg);
          delete appmsg;
          //}
          break;
        }

      case OS_KILL:
        //delete app;
        // app = NULL;

        break;

      default:
        std::cout << "Core id: " << myid.to_string() << "\n";
        opp_error("Core: unknown OS message type received.");

        }

      delete cmsg;

    }
  else
    { //it's a scheduled message to send out

      send(cmsg, outgate);

    }
}

// --- sstmac::sw::AMI pure virtuals ---- //
void
Core::executeKernel(int func, boost::shared_ptr<sstmac::sw::sst_message> data)
{
  switch (func)
    {
  case sstmac::sw::AMI::AMI_COMM_SEND:
    {
      payloadpod ppod(data);

      ApplicationMsg* cmsg = new ApplicationMsg();
      cmsg->setMsgType(APP);
      cmsg->setMsgcontent(ppod);
      Address* froma = (Address*) data->fromaddr_.get();
      Address* toa = (Address*) data->toaddr_.get();
      cmsg->setFromAddr(*froma);
      cmsg->setToAddr(*toa);
      cmsg->setByteLength(data->get_byte_length());

      dbg << "Core(" << myid.to_string() << "): sending to " << toa->to_string()
          << "\n";

      send(cmsg, outgate);
      break;
    }

  case sstmac::sw::AMI::AMI_COMP_TIME:
    {



      //grab the first core available
      map<cMessage*, boost::shared_ptr<sstmac::sw::sst_message> >::iterator it;
      bool found = false;
      cMessage* compmsg = NULL;
      for (it = computeMessages.begin(); it != computeMessages.end() && !found; it++)
        {
          if (!it->first->isScheduled())
            {
              found = true;
              compmsg = it->first;
            }
        }

      if (!found) //put in suspended queue
        {
          dbg << "Core(" << myid.to_string() << "): suspending compute request"
                        << "\n";
          suspendedQ.push(data);
        }
      else
        {

          dbg << "Core(" << myid.to_string() << "): computing for time " << data->get_time()
                        << "\n";

          computeMessages[compmsg] = data;
          scheduleAt(simTime() + data->get_time(), compmsg);
          // opsys->block(data->get_key());
        }
      break;
    }

  default:
    std::cout << "Kernel number: " << func << endl;
    opp_error("Core: received a kernel which I don't know how to execute");

    }

}

timestamp
Core::now()
{
  return simTime();
}

bool
Core::kernelSupported(int func)
{
  switch (func)
    {
  case sstmac::sw::AMI::AMI_COMM_SEND:
    return true;

  case sstmac::sw::AMI::AMI_COMM_NIC_SEND_ACK:
    return true;

  case sstmac::sw::AMI::AMI_COMP_TIME:
    return true;

  default:
    return false;

    }
}
// ------------------------------------ //

*/
