/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/backends/omnetpp/node/processor/fakenic.h>
#include <sstmac/backends/omnetpp/node_m.h>
#include <sstmac/backends/omnetpp/node/processor/abstractnodeprocessor.h>

#include <sstmac/common/errors.h>

namespace sstmac
{
  namespace omnetpp
  {

    // Pointer types.

    using namespace sstmac::hw;

    //
    // Hi.
    //
    fakenic::fakenic(const node::ptr &parent, abstractnodeprocessor* op,
        int64_t num_nodes) :
        networkinterface(parent, sstmac::hw::interconnect::ptr()), opp_parent_(op)

    {
        if (!spy_)
          {
            spy_ = stat_spyplot::construct("<stats> spyplot", "network_spyplot.csv");
            register_stat(spy_);

            spy_bytes_ = stat_spyplot::construct("<stats> spyplot", "network_spyplot_bytes.csv");
            register_stat(spy_bytes_);
          }
    }

    //
    // Hello.
    //
    fakenic::ptr
    fakenic::construct(const node::ptr &parent, abstractnodeprocessor* op, int64_t num_nodes)
    {
      return fakenic::ptr(new fakenic(parent, op, num_nodes));
    }

    //
    // Goodbye.
    //
    fakenic::~fakenic() throw ()
        {
        }

    //
    // Schedule a send operation to happen at the given time.
    //
    void
    fakenic::send(const nodeaddress::ptr &recver,
        const sst_message::ptr &payload, const eventhandler::ptr &completion)
    {
      SSTMAC_DEBUG << "fakenic::send(" << recver << ", " << payload->to_string()
              << ", " << completion << ")\n";

      if (payload == 0)
        {
          throw nullerror("fakenic::send: null payload pointer.");
        }


      timestamp delayed = now(); //don't model delay for now

      payloadpod
      ppod(payload);

      ApplicationMsg* cmsg = new ApplicationMsg();
      cmsg->setMsgType(APP);
      cmsg->setMsgcontent(ppod);
      omnetpp_address::ptr froma = boost::dynamic_pointer_cast<omnetpp_address>(
          payload->fromaddr_);
      omnetpp_address::ptr toa = boost::dynamic_pointer_cast<omnetpp_address>(
          payload->toaddr_);
      cmsg->setFromAddr(froma);
      cmsg->setToAddr(toa);
      cmsg->setByteLength(payload->get_byte_length());
      cmsg->setKind(payload->get_kind() + 1); //add 1, because 0 is reserved for Processor/management type stuff like launch

      SSTMAC_DEBUG << "fakenic(" << opp_parent_->myid->to_string()
            << "): sending to " << toa->to_string() << "\n";

      opp_parent_->send(cmsg, opp_parent_->outgate);

    }

    networkinterface::ptr
    fakenic::clone(const node::ptr &parent,
        const sstmac::hw::interconnect::ptr &interconn) const
    {
      throw unimplementederror("fakenic::clone(...) should not be called");
    }

    void
    fakenic::handle(const sst_message::ptr &msg)
    {
      SSTMAC_DEBUG << "fakenic::handle - received message \n";

      parent_->handle(msg);
    }

  }
} // end of namespace sstmac.
