/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_OMNETPP_NODE_PROCESSOR_MEDIUM_SERIALIZER_H_INCLUDED
#define SSTMAC_BACKENDS_OMNETPP_NODE_PROCESSOR_MEDIUM_SERIALIZER_H_INCLUDED

#include <omnetpp.h>

#include <sstmac/common/messages/sst_serializer.h>

namespace sstmac
{
  namespace omnetpp
  {

    class medium_serializer : public sstmac::sst_serializer
    {
    public:
      medium_serializer(cCommBuffer* b) :
          buffer(b)
      {
      }

      virtual boost::intrusive_ptr<nodeaddress>
      get_deser_addr();

      virtual
      ~medium_serializer();

      virtual void
      pack(int i);
      virtual void
      unpack(int &i);

      virtual void
      pack(long int i);
      virtual void
      unpack(long int &i);

      virtual void
      pack(long long int i);
      virtual void
      unpack(long long int &i);

      virtual void
      pack(long long unsigned int i);
      virtual void
      unpack(long long unsigned int &i);

      virtual void
      pack(size_t i);
      virtual void
      unpack(size_t &i);

      virtual void
      pack(std::string i);
      virtual void
      unpack(std::string &i);

      virtual void
      pack(sstmac::timestamp i);
      virtual void
      unpack(sstmac::timestamp &i);

      virtual void
      pack(double i);

      virtual void
      unpack(double &i);

      virtual void
      pack(bool i);

      virtual void
      unpack(bool &i);

      virtual void
      pack(richpair<int, int> i);
      virtual void
      unpack(richpair<int, int> &i);

    protected:
      cCommBuffer* buffer;
    };
  }
}
#endif
