/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_PROCESSOR_EVENTDATA_H_INCLUDED
#define SSTMAC_HARDWARE_PROCESSOR_EVENTDATA_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/ptr_type.h>
#include <string>
#include <map>
#include <stdint.h>

namespace sstmac {

  /**
   * Input for processor models that use
   * performance counter data. Is basically just a map
   * that maps string keys to integer values.
   * Keys are defined in the libraries that use them.
   */
  class eventdata : public sst_message {


    /**
     * Private constructor
     * @param libn The library name which is notified when this event is done
     * @param t The default multi-use time parameter
     */
    eventdata(std::string libn, timestamp t);


  public:
    typedef boost::intrusive_ptr<eventdata> ptr;
    typedef boost::intrusive_ptr<const eventdata> const_ptr;

    /**
      * Construct a new eventdata object.
      * @param libn The library name which is notified when this event is done
      * @param t The default multi-use time parameter
      * @return A smart pointer to an eventdata object.
      */
    static eventdata::ptr
      construct(std::string libn, timestamp t = timestamp(0));

    virtual ~eventdata() {}

    /**
     *
     * @param key the key
     * @param val value to associate with the given key
     */
    void
    set_event(std::string key, unsigned long long int val)
    { data[key]=val; }


    /**
      * Get event data
      * @param key the key
      * @return value associated with given key
      */
    unsigned long long int get_event(std::string key);


    /**
      * Test for event key presence
      * @param key to test
      * @return true if this eventdata has the given key
      */
    bool has_event(std::string key);

  private:

    std::map<std::string, unsigned long long int> data;

  };

}  // end of namespace sstmac

#endif
