/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/messages/sst_message.h>
#include <sstmac/common/errors.h>

namespace sstmac {




sst_message::sst_message(std::string lib, timestamp t,
    sst_message::MESSAGE_TYPES ty, int k) :
    lib_name_(lib), time_(t), msgtype_(ty), kind_(k), needs_ack_(true)
{

}

sst_message::sst_message(std::string libn, timestamp t,
    nodeaddress::ptr from, nodeaddress::ptr to,
    sst_message::MESSAGE_TYPES ty, int k) :
    lib_name_(libn), time_(t), msgtype_(ty), fromaddr_(from), toaddr_(to), kind_(k), needs_ack_(true)
{

}

sst_message::~sst_message()
{
  // std::cout << "calling sst_message destructor now" << std::endl;
}


//
// Allocate an object of this type.
//

sst_message::ptr
sst_message::construct(std::string lib, timestamp t, MESSAGE_TYPES ty, int k)
{
  sst_message::ptr retval(new sst_message(lib, t, ty, k));

  return retval;
}

sst_message::ptr
sst_message::construct(std::string lib, timestamp t, nodeaddress::ptr from, nodeaddress::ptr to, MESSAGE_TYPES ty, int k)
{
  sst_message::ptr retval(new sst_message(lib, t, from, to, ty, k));

  return retval;
}

void
sst_message::serialize(sst_serializer* ser)
{
  // std::cout << "in sst_message serialize" << std::endl;
  ser->pack(lib_name_);
  ser->pack(time_);
  fromaddr_->serialize(ser);
  toaddr_->serialize(ser);

  //std::cout << "completed sst_message serialize" << std::endl;
}
sst_message::ptr
sst_message::deserialize(sst_serializer* ser)
{
  sst_message::ptr ret = sst_message::construct();
  ret->do_deser(ser);

  return ret;

}

void
sst_message::do_deser(sst_serializer* ser)
{
  ser->unpack(lib_name_);
  ser->unpack(time_);
  fromaddr_ = ser->get_deser_addr();
  fromaddr_->deserialize(ser);
  toaddr_ = ser->get_deser_addr();
  toaddr_->deserialize(ser);
}

} // end of namespace sstmac
