

/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_COMMON_PTR_TYPE_H_INCLUDED
#define SSTMAC_COMMON_PTR_TYPE_H_INCLUDED

namespace sstmac{
  class ptr_type;
  
}

namespace boost
{
  void intrusive_ptr_add_ref(sstmac::ptr_type * p);
  void intrusive_ptr_release(sstmac::ptr_type * p);
  
  void intrusive_ptr_add_ref(const sstmac::ptr_type * p);
  void intrusive_ptr_release(const sstmac::ptr_type * p);
};

#include <typeinfo>
#include <boost/intrusive_ptr.hpp>
#include <map>
#include <string>

namespace sstmac {
  /**
   * Base class to implement functions for intrusive_ptr.  Any child class must use
   * the construct mechanism and never create a local instance or un-smart pointer.
   */
  class ptr_type {
  private:
      mutable long  references;
      friend void ::boost::intrusive_ptr_add_ref(ptr_type * p);
      friend void ::boost::intrusive_ptr_release(ptr_type * p);

      friend void ::boost::intrusive_ptr_add_ref(const ptr_type * p);
      friend void ::boost::intrusive_ptr_release(const ptr_type * p);


    public:
      typedef boost::intrusive_ptr<ptr_type> ptr;

      ptr_type() : references(0) {//std::cout << "allocating for " << this << std::endl;
        }   // initialize references to 0

      virtual ~ptr_type();

      virtual std::string to_string() const = 0;
  };



} // end of namespace sstmac


namespace boost
{
 inline void intrusive_ptr_add_ref(sstmac::ptr_type * p)
  {
    // increment reference count of object *p
   //std::cout << "Adding! to " << p  << ", which is a " << typeid(p).name()<< std::endl;
    ++(p->references);

  }



 inline void intrusive_ptr_release(sstmac::ptr_type * p)
  {
   // decrement reference count, and delete object when reference count reaches 0

   if (--(p->references) == 0){
      // std::cout << "Deleting! a " << p->to_string() << " at address " << p << std::endl;

     delete p;
   }

  }
 inline void intrusive_ptr_add_ref(const sstmac::ptr_type * p)
   {
   //std::cout << "Adding! to " << p  << ", which is a " << typeid(p).name()<< std::endl;
   ++(p->references);
   }



  inline void intrusive_ptr_release(const sstmac::ptr_type * p)
   {
    if (--(p->references) == 0){
       // std::cout << "Deleting const! a " << p->to_string() << " at address " << p << std::endl;

        delete p;
      }
   }


} // namespace boost


#endif
