/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/common/stats/stat_time_window.h>

namespace sstmac {

using namespace std;

timestamp stat_time_window::TIME_INTERVAL;
bool stat_time_window::interval_is_set_ = false;
int stat_time_window_sum_duration::duration_record::ID_CNT = 0;

void
stat_time_window_sum::record(timestamp t, double val)
{
  if (is_active() && check_open())
    {
      val_ += val;

      if (t >= current_win_time_ + TIME_INTERVAL)
        { //we hit a new window
          do
            {
              (*this) << current_win_time_ << ", " << val_ << "\n";
              current_win_++;
              val_ = 0;
              current_win_time_ += TIME_INTERVAL;
              // std::cout << "t=" << t << ", current_win_time=" << current_win_time_ << "\n";
            }
          while (t >= current_win_time_ + TIME_INTERVAL);
        }

    }
}

void
stat_time_window_avg::record(timestamp t, double val)
{
  if (is_active() && check_open())
    {

      while (t >= current_win_time_ + TIME_INTERVAL) //back fill
        {

          (*this) << current_win_time_ << ", "
              << (numvals_ > 0 ? (val_ / numvals_) : 0)<<"\n";

          val_ = 0;
          numvals_ = 0;

          current_win_time_ += TIME_INTERVAL;
          // std::cout << "t=" << t << ", current_win_time=" << current_win_time_ << "\n";

        }

      val_ += val;
      numvals_++;

    }
}

void
stat_time_window_time_avg::record(timestamp t, double val)
{
  if (is_active() && check_open())
    {

      timestamp next = current_win_time_ + TIME_INTERVAL;
      timestamp proxy = (t > next) ? (next) : (t);
      timestamp t1 = proxy - last_update_;
      timestamp t2 = last_update_ - current_win_time_;
      timestamp t3 = proxy - current_win_time_;
      val_ = (pending_val_ * t1 + val_ * t2).sec() / t3.sec();

      while (t >= current_win_time_ + TIME_INTERVAL) //back fill
        {

          (*this) << current_win_time_ << ", " << val_ << "\n";

          current_win_time_ += TIME_INTERVAL;
          // std::cout << "t=" << t << ", current_win_time=" << current_win_time_ << "\n";
          val_ = pending_val_;

        }

      if (last_update_ < current_win_time_)
        last_update_ = current_win_time_;

      pending_val_ = val;

      last_update_ = t;

    }
}

void
stat_time_window_sum_duration::record(timestamp t, double val, timestamp dur)
{
  if (is_active() && check_open())
    {

      while (t >= current_win_time_ + TIME_INTERVAL) //back fill
        {

          map<int, duration_record> templst;
          templst.insert(records_.begin(), records_.end());
          map<int, duration_record>::iterator it, end = templst.end();

          double added = 0;
          for (it = templst.begin(); it != end; it++)
            {

            timestamp sub(0);
            if (it->second.start_ >= current_win_time_)
              {
                sub += (it->second.start_ - current_win_time_);
              }

            if (it->second.end_ < current_win_time_ + TIME_INTERVAL)
              {
                sub += ((current_win_time_ + TIME_INTERVAL)
                    - (it->second.end_) );
                records_.erase(it->second.id_);
              }

            added += (it->second.val_ * (TIME_INTERVAL - sub).sec());

            //std::cout << "at t=" << t << " -- added duration record: start=" << it->start_ << ", val=" << it->val_ << ", end=" << it->end_ << std::endl;
          }

        (*this) << current_win_time_ << ", " << added << "\n";

        current_win_time_ += TIME_INTERVAL;

      }

    timestamp valend = t + dur;
    duration_record rec(val, t, valend);
    records_[rec.id_] = (rec);

  }
}
} // end of namespace sstmac
