/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/failure/failable.h>
#include <sstmac/common/eventmodule.h>

namespace sstmac
{
  namespace hw
  {

    stat_sum::ptr failable::uses_while_failed_;
    debug failable::dbg_;
    long failable::failable_count_ = 0;


    void
    failable::init_failable(failure_model::ptr mo,
        const boost::intrusive_ptr<eventmodule> &parentmodel)
    {
      model_ = mo;
      has_failed_ = (false);

      if (!uses_while_failed_)
        {
          uses_while_failed_ = stat_sum::construct("<stats> failures",
              "failures.csv");
          parentmodel->register_stat(uses_while_failed_);
        }

      if (!dbg_.is_real())
        {
          dbg_ = debug("<debug> failures");
          logger::register_logger(&dbg_);
        }


      model_->next_failure(parentmodel);
    }


  }
}

