/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/failure/failure_rate_model.h>
#include <sstmac/common/eventmodule.h>
#include <sstmac/hardware/failure/failable.h>
namespace sstmac
{
  namespace hw
  {
    long failure_rate_model::rand_max = (1 << 29);
    RNG::UniformInteger::ptr failure_rate_model::rng_;

    void
    failure_rate_model::next_failure(
        boost::intrusive_ptr<eventmodule> parentmodel)
    {
      timestamp fail_time = parentmodel->now() + timestamp(genexp());
      SSTMAC_DEBUG << to_string() << ": next failure is at " << fail_time
            << "\n";
      failure_event::ptr evnt = failure_event::construct();
      parentmodel->schedule(fail_time, parentmodel, evnt);
    }

    void
    failure_rate_model::propagate()
    {
      std::map<failable::ptr, double>::iterator it, end = children_.end();
      for (it = children_.begin(); it != end; it++)
        {
          double rnd = urand();
          if (rnd < it->second)
            {
              it->first->parent_failed();
            }
        }
    }

  }
}
