/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_HARDWARE_FAILURE_GENERIC_FAILURE_COMPONENT_H_INCLUDED
#define SSTMAC_HARDWARE_FAILURE_GENERIC_FAILURE_COMPONENT_H_INCLUDED

#include <sstmac/hardware/failure/failable.h>
#include <sstmac/common/eventmodule.h>

namespace sstmac
{
  namespace hw
  {

    class generic_failure_component : public eventmodule, public failable
    {

    protected:
      generic_failure_component(std::string name, long id) : eventmodule(), failable(), name_(name), uid_(id)
      {

        //init_failable(l, self_);
      }

    public:
      typedef boost::intrusive_ptr<generic_failure_component> ptr;

      virtual
      ~generic_failure_component() { }

      static ptr
      construct(std::string n, long id)
      {
        return ptr(new generic_failure_component(n, id));
      }

      virtual std::string
      to_string() const
      {
        return "failure_component_" + name_;
      }

      virtual void
      handle(const sst_message::ptr &msg)
      {
        failure_occurred(msg, self_);
      }

    protected:
      std::string name_;
      long uid_;
    };

  }
}
#endif
