/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_NETWORK_CONGESTION_SHAREDBUFFER_DEPARTURE_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_CONGESTION_SHAREDBUFFER_DEPARTURE_H_INCLUDED

#include <sstmac/hardware/network/congestion/sharedbuffer.h>
#include <sstmac/common/eventhandler.h>
#include <sstmac/hardware/network/topology/topology.h>

namespace sstmac { namespace hw  {

/**
 * Handles the departure of the tail of a flow from a particular vertex.
 */
class sharedbuffer::departure: public eventhandler {
        /// sharedbuffer parent
        sharedbuffer::ptr parent_;
        /// The departing flow.
        flow_id_t fid_;
        /// The vertex the flow is departing from.
        vertex_index v_;

protected:
        /// Construct a new departure event.
        departure(sharedbuffer::ptr parent, flow_id_t fid,
                        vertex_index v);

public:
        typedef boost::intrusive_ptr<departure> ptr;

        /// Construct a new departure event.
        static departure::ptr
        construct(sharedbuffer::ptr parent, flow_id_t fid,
                        vertex_index v);

        /// Destructor.
        virtual ~departure() throw ();

        /// Handle the departure of flow fid_ from vertex v_.
        virtual void handle(const sst_message::ptr &msg);

        virtual std::string to_string() const {
                return "sharedbuffer::departure";
        }

};

} } // end of namespace sstmac

#endif
