/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


// product.h: Interface for product networks.
//
// Author: Curtis Janssen <cljanss@ca.sandia.gov>

#ifndef SSTMAC_HARDWARE_NETWORK_TOPOLOGY_PRODUCT_H_INCLUDED
#define SSTMAC_HARDWARE_NETWORK_TOPOLOGY_PRODUCT_H_INCLUDED

#include <memory>
#include <sstmac/hardware/network/topology/topology.h>

namespace sstmac { namespace hw  {

  /** The product network contructs a new network by taking the product of two
      networks. */
  class product: public topology {
  private:
    topology::ptr n1_, n2_;
  protected:
    // called by derived class CTOR
    void assemble(const topology::ptr& n1, const topology::ptr& n2);

  protected:
    //product(TiXmlHandle) {}
    product() {}

  public:
    typedef boost::intrusive_ptr<product> ptr;

    static product::ptr construct();
    virtual ~product();

    int64_t diameter() const;
    double a() const;
    int64_t b() const;

    int64_t shortest_path_length(const vertex &v1, const vertex &v2) const;
    void shortest_path(const vertex &v1, const vertex &v2, walk &) const;
    void next_hop(vertex_index loc, vertex_index dst,
                  std::vector<lwedge> &nxt) const;

    bool vertex_symmetric() const;
  };

} } //end of namespace sstmac

#endif
