/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */


// walk.cc: Implementation for walks through a graph.
//
// Author: Curtis Janssen <cljanss@ca.sandia.gov>

#include <sstmac/hardware/network/topology/walk.h>

using std::vector;
using std::ostream;

namespace sstmac { namespace hw  {

void
walk::init(const vector<vertex_index> &v)
{
  e_.clear();
  for (int i=1; i<v.size(); i++) {
      e_.push_back(lwedge(v[i-1],v[i]));
    }
}

std::ostream &
operator << (std::ostream &o, const walk&w)
{
  w.print(o);
  return o;
}

void
walk::print(ostream &o) const
{
  if (e_.size() == 0) return;

  o << e_[0].from();
  for (int i=0; i<e_.size(); i++) {
      o << " (" << e_[i].id() << ")-> " << e_[i].to();
    }
}

const vertex_index &
walk::v(int i) const
{
  if (i==0) return e_[i].from();
  return e_[i-1].to();
}

} } //end of namespace sstmac
