/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_DEGRADEDNODE_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_DEGRADEDNODE_H_INCLUDED

#include <sstmac/hardware/node/simplenode.h>

#include <sstmac/hardware/processor/simpleprocessor.h>
#include <sstmac/hardware/processor/instructionprocessor.h>
#include <sstmac/hardware/processor/computescheduler.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/hardware/node/node.h>
#include <sstmac/common/nocopy.h>

#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/units.h>

#include <map>

//#include <sstmac/processor/processor_config.h>

namespace sstmac
{
  namespace hw
  {

    /**
     * A simple node that is degraded, cutting its performance by a factor
     */
    class degradednode : public simplenode, virtual public nocopy
    {

    protected:
      degradednode(double fac, const sim_parameters::ptr &params) :
          simplenode(params), factor_(fac)
      {

      }

      degradednode(const sim_parameters::ptr &params) :
          simplenode(params)
      {
        factor_ = params->get_optional_double_param("node_degraded_factor",
            1.0);
      }

      double factor_;

    public:
      typedef boost::intrusive_ptr<degradednode> ptr;

      static ptr
      construct(sim_parameters::ptr params)
      {
        return ptr(new degradednode(params));
      }

      static ptr
      construct(sim_parameters::ptr params, double factor)
      {
        return ptr(new degradednode(factor, params));
      }

      virtual void
      initialize(nodeaddress::ptr my_id, eventmanager::ptr man);

      virtual
      ~degradednode()
      {

      }

    };

  }
} // end of namespace sstmac

#endif
