/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_detailednode_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_detailednode_H_INCLUDED

#include <sstmac/hardware/node/simplenode.h>

namespace sstmac
{
  namespace hw
  {
    /**
     * A node with a slightly more detailed memory model
     */
    class detailednode : public simplenode
    {

    protected:
      detailednode(const sim_parameters::ptr &params);

      timestamp proc_link_lat_;
      double proc_link_bw_;

      int num_mc_;
      int num_sock_;

    public:
      typedef boost::intrusive_ptr<detailednode> ptr;

      static ptr
      construct(sim_parameters::ptr params)
      {
        ptr ret = ptr(new detailednode(params));

        return ret;
      }

      virtual node::ptr
      clone() const;

      virtual void
      initialize(nodeaddress::ptr my_id, eventmanager::ptr man);


    };

  }
} // end of namespace sstmac

#endif
