/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_SIMPLENODE_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_COMPONENTS_NODE_SIMPLENODE_H_INCLUDED

#include <sstmac/hardware/processor/simpleprocessor.h>
#include <sstmac/hardware/processor/instructionprocessor.h>
#include <sstmac/hardware/processor/computescheduler.h>
#include <sstmac/hardware/memory/memorymodel.h>
#include <sstmac/hardware/disk/vesper/disk_controller.h>
#include <sstmac/hardware/nic/networkinterface.h>
#include <sstmac/hardware/node/node.h>
#include <sstmac/common/nocopy.h>

#include <sstmac/common/sim_parameters.h>
#include <sstmac/common/units.h>

#include <map>

//#include <sstmac/processor/processor_config.h>

namespace sstmac
{
  namespace hw
  {

    // Forward declaration of important types.
    //class procscheduler;

    /**
     * A stand-in node.  This node type passes all requests directly to
     * the simulator and has no resource contention (infinite processors),
     */
    class simplenode : public node, virtual public nocopy
    {

    protected:
      simplenode(const sim_parameters::ptr &params);

      timestamp memory_latency_;
      double memory_bandwidth_;
      double freq_;
      int cores_;
      timestamp preempt_;

      sstmac::hw::simpleprocessor::ptr proc_model_;
      sstmac::hw::instructionprocessor::ptr proc_inst_;
      sstmac::hw::computescheduler::ptr compute_sched_;
      sstmac::hw::memorymodel::ptr mem_model_;
      sstmac::hw::disk_controller::ptr disk_;

      nodeaddress::ptr nppfs_server_;

      virtual bool
      has_failed();
      virtual void
      reset_failed();

    public:
      typedef boost::intrusive_ptr<simplenode> ptr;

      static ptr
      construct(sim_parameters::ptr params)
      {
        ptr ret = ptr(new simplenode(params));

        return ret;
      }

      // ------- AMI functions --------- //
      virtual bool
      kernel_supported(int func);

      virtual timestamp
      now();

      virtual void
      execute_kernel(int func, sst_message::ptr data);

      virtual ptr_type::ptr
      execute_kernel(int func);

      // --------------------- //

      virtual node::ptr
      clone() const;

      virtual void
      initialize(nodeaddress::ptr my_id, eventmanager::ptr man);

      virtual std::list<failable::ptr>
      get_failure_components();

      //virtual void
      //init_failure_model(double l_proc, double l_mem, double l_disk, double l_nic, failable::ptr ps, double p_ps_node);
    };

  }
} // end of namespace sstmac

#endif
