/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/hardware/processor/processor.h>

namespace sstmac { namespace hw 
{

  using namespace std;
  
  debug processor::dbg_;

  stat_sum::ptr processor::stat_pwr_;
  stat_time_window_sum_duration::ptr processor::stat_energy_;
  stat_time_window_sum::ptr processor::stat_proc_;

  processor::processor(processor_config::ptr cfg,
      const eventhandler::ptr &completion, computescheduler::ptr sched) :
      proc_cfg(cfg), completion_(completion), csched_(sched)
  {


    if(!dbg_.is_real()){
         dbg_ = debug("<debug> processor | proc");
         logger::register_logger(&dbg_);
     }

    if (!stat_pwr_){
        stat_pwr_ = stat_sum::construct("<stats> proc | processor",
            "stats_power_proc.csv");
       sched->register_stat(stat_pwr_);
    }

    if (!stat_energy_){
        stat_energy_ = stat_time_window_sum_duration::construct("<stats> proc | processor",
            "stats_proc_energy.csv");
        sched->register_stat(stat_energy_);
    }

    if (!stat_proc_){
        stat_proc_ = stat_time_window_sum::construct("<stats> proc | processor",
            "stats_proc.csv");
        sched->register_stat(stat_proc_);
    }
  }

  processor_config::ptr
  processor::getProcCfg()
  {
    return proc_cfg;
  }



  void
  processor::request_compute(const timestamp& comp,
      const sst_message::ptr& to_deliver, long thread_id)
  {
    SSTMAC_DEBUG << "processor::request_compute for " << comp << "\n";
    //grab the first core available

    eventhandler::ptr hand_cast = boost::dynamic_pointer_cast<eventhandler>(ptr(this));

    if(!hand_cast)
      throw nullerror("processor::request_compute: error casting self to eventhandler");

    csched_->schedule_computation(comp, to_deliver, hand_cast, thread_id);

  }

  void
  processor::handle(const sst_message::ptr &msg)
  {
    SSTMAC_DEBUG << "[t=" << csched_->now() << "] -- Proc(" << "): done computing " << "\n";


    completion_->handle(msg);
  }

} } //end of namespace sstmacs
