/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/debug.h>
#include <sstmac/common/errors.h>
#include <sstmac/hardware/processor/simpleprocessor.h>
//#include <sstmac/processor/eventdata.h>
#include <sstmac/hardware/processor/simpleprocessor-config.h>
//#include <sstmac/processor/instructions.h>
#include <iostream>

namespace sstmac { namespace hw 
{
  

//
// Constructor
//
  simpleprocessor::simpleprocessor(
      const simpleprocessor_config::ptr& cfg,
      const eventhandler::ptr &completion,
      computescheduler::ptr sched) :
      processor(cfg, completion, sched), config_(cfg)
  {


    if(stat_pwr_->is_active())
      stat_pwr_->record(config_->get_pwr_static());

  }

//
// Construct a new processor with the given average service times
//
  simpleprocessor::ptr
  simpleprocessor::construct(
      const simpleprocessor_config::ptr& cfg,
      const eventhandler::ptr &completion,
      computescheduler::ptr sched)
  {
    simpleprocessor::ptr retval(
        new simpleprocessor(cfg, completion, sched));

    return retval;
  }

  processor::ptr
  simpleprocessor::clone() const
  {

    simpleprocessor::ptr retval(
        new simpleprocessor(config_, completion_, csched_));
    return retval;
  }

  void
  simpleprocessor::compute(const sst_message::ptr& inst, long thread_id)
  {
    SSTMAC_DEBUG << "simpleprocessor::compute(" << inst->to_string() << ")\n";

    if(stat_energy_->is_active()){
        stat_energy_->record(csched_->now(), inst->get_time().sec() * config_->get_per_sec(), inst->get_time());
        stat_proc_->record(csched_->now(), inst->get_time().sec() * config_->get_fps());
    }

    // std::cerr << "simpleproc: compute: returning time " << data->comptime.sec() << endl;
    request_compute(inst->get_time(), inst, thread_id);
  }

} } // end of namespace sstmac
