/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */


#ifndef SSTMAC_SOFTWARE_AMI_AMI_H_INCLUDED
#define SSTMAC_SOFTWARE_AMI_AMI_H_INCLUDED

#include <sstmac/common/sstmac-config.h>

#include <sstmac/common/errors.h>

#include <sstmac/common/messages/sst_message.h>

namespace sstmac { namespace sw
{

  class AMI
  {
  public:
    enum AMI_CAT
    {
      AMI_COMPUTE = 0, AMI_COMM = 1, AMI_DISP = 2, AMI_QUERY = 3
    };

    enum AMI_COMM_FUNC
    {
      AMI_COMM_SEND = 10101, //pretty much have to implement this one

      AMI_COMM_BCAST,
      AMI_COMM_BARRIER,

      AMI_COMM_NIC_SEND_ACK   //this is sent back to the processor after the message has left the source

    };

    enum AMI_COMP_FUNC
    {

      AMI_COMP_TIME = 67, //the basic compute-for-some-time

      AMI_COMP_INSTR,
      AMI_COMP_MEMACCESS,
      AMI_COMP_DISKACCESS,
      AMI_COMP_STOCH,
      AMI_COMP_SLEEP,
      AMI_COMP_REGISTER_THREAD

    };

    //for changing graphics display colors, if possible
    enum AMI_DISP_FUNC
    {
      AMI_DISP_BORDER = 231,
      AMI_DISP_FILL
    };
    
    enum AMI_QUERY_FUNC
    {
      AMI_QUERY_TOPOLOGY = 999,
      AMI_QUERY_ADDRESS = 1000,
      AMI_QUERY_MACHINE = 1001
    };

    virtual ~AMI() {}

    virtual void
    execute_kernel(int func, sst_message::ptr data) = 0;

    //this is also a form, used for things like querying
    virtual ptr_type::ptr
    execute_kernel(int func) { return ptr_type::ptr(); }

    virtual bool
    kernel_supported(int func) = 0;

    virtual timestamp
    now() = 0; // for convenience, why not


  };

} } //end of namespace sstmac

#endif
