/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */


#ifndef SSTMAC_SOFTWARE_AMI_ENVIRONMENT_H_INCLUDED
#define SSTMAC_SOFTWARE_AMI_ENVIRONMENT_H_INCLUDED



#include <sstmac/software/process/taskid.h>
#include <sstmac/common/nodeaddress.h>



#include <sstmac/common/ptr_type.h>

#include <sstmac/common/sstmac-config.h>

#include <sstmac/common/messages/sst_serializer.h>
#ifdef SSTMAC_HAVE_BOOST_1_37
#include <boost/unordered_map.hpp>
#else
#include <map>
#endif

namespace sstmac { namespace sw
{

  class environment : public ptr_type
  {
  public:
    typedef boost::intrusive_ptr<environment> ptr;
    typedef boost::intrusive_ptr<const environment> const_ptr;


    // ------- constructor and boost stuff ----------
  public:
    static environment::ptr
    construct();

    virtual
    ~environment();

  protected:
    environment();



  protected:
#ifdef SSTMAC_HAVE_BOOST_1_37
    typedef boost::unordered_map<taskid, nodeaddress::ptr> map_type;
#else
    typedef std::map<taskid, nodeaddress::ptr> map_type;
#endif

    map_type mapping_;

    std::list<nodeaddress::ptr> extras_;

  public:

    virtual std::string to_string() const{
            return "environment";
          }

    const nodeaddress::ptr
    get_node_address(taskid &s) const
    {
      nodeaddress::ptr ret = mapping_.find(s)->second;
      return ret;
    }

    size_t
    get_app_count() const
    {
      return mapping_.size();
    }

    void
    add_item(taskid& s, const nodeaddress::ptr &n)
    {
      mapping_[s] = n;

    }

    void
    removeItem(taskid& s, const nodeaddress::ptr &n)
    {
      mapping_.erase(s);

    }

    void
    add_extra(nodeaddress::ptr a){
      extras_.push_back(a);
    }

    std::list<nodeaddress::ptr>
    get_extras() const{
      return extras_;
    }

    void serialize(sst_serializer* ser);
    static environment::ptr deserialize(sst_serializer* ser);
  };

} } // end of namespace sstmac

#endif
