/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_ACTOR_ACTORPATTERN_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_ACTOR_ACTORPATTERN_H_INCLUDED

#include <sstmac/software/libraries/actor/actormsg.h>
#include <sstmac/common/ptr_type.h>

namespace sstmac { namespace sw {

  // Forward declaration of important types.


  /**
   * Base type for pattern-matching on incoming messages.
   * Any test or recv call is pattern matched using a actorpattern.
   */
  class actorpattern : public ptr_type {
  public:

          typedef boost::intrusive_ptr<actorpattern> ptr;

    /// A nested pattern type that matches anything.
    class anypattern;
    /// A static object to match everything.
    static ptr any_;

  public:

    virtual std::string to_string() const{
            return "actorpattern";
          }

    /// Goodbye.
    virtual ~actorpattern() {}

    /// Is this a match?
    /// \return true for a match.
    virtual bool matches(const actormsg::ptr& msg) const = 0;

    /// A special magical match that always returns true for a non-NULL message.
    static ptr any();
  };

} } // end of namespace sstmac.

#endif
