/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_ACTOR_ACTORWORLD_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_ACTOR_ACTORWORLD_H_INCLUDED

#include <sstmac/software/ami/environment.h>
#include <sstmac/common/ptr_type.h>

namespace sstmac
{
  namespace sw
  {

    class actorworld : public ptr_type
    {

      environment::const_ptr env_;

      actorworld(environment::const_ptr e) :
          env_(e)
      {

      }

    public:
      typedef boost::intrusive_ptr<actorworld> ptr;

      static actorworld::ptr
      construct(environment::const_ptr e)
      {
        actorworld::ptr ret(new actorworld(e));

        return ret;
      }

      virtual std::string
      to_string() const
      {
        return "actorworld";
      }

      size_t
      size()
      {
        return env_->get_app_count();
      }

      nodeaddress::ptr
      get_addr(taskid t)
      {
        return env_->get_node_address(t);
      }

      actorid
      get_base_actorid(softwareid sid)
      {
        return actorid(get_addr(sid.task_), 0, sid.app_);
      }
    };

  }
} //end of namespace sstmac

#endif
