/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <math.h>

#include <sstmac/software/libraries/compute/lib_compute_loops.h>

#include <sstmac/software/process/key.h>

namespace sstmac
{
  namespace sw
  {

    int lib_compute_loops::mem_op_size_ = 16;

    double lib_compute_loops::mem_line_ratio_ = 0.8;
    double lib_compute_loops::flop_line_ratio_ = 0.8;




    void
    lib_compute_loops::compute_loop(int from, int to, int numlines)
    {
      int loop = to - from + 1; //+1 for fortran loops

      doing_memory_ = true;
      lib_compute_memmove::compute_block_access(
          loop * mem_line_ratio_ * numlines * mem_op_size_);
      doing_memory_ = false;

      eventdata::ptr inst = eventdata::construct(get_lib_name(), timestamp(0));
      inst->set_event(INST_FLOP_LABEL, loop * flop_line_ratio_ * numlines);
      lib_compute_inst::compute_inst(inst);
    }

    void
    lib_compute_loops::compute_loop2(int from1, int to1, int from2, int to2,
        int numlines)
    {
      int loop1 = to1 - from1 + 1; //+1 for fortran loops
      int loop2 = to2 - from2 + 1;
      int loop = loop1 * loop2;

      doing_memory_ = true;
      lib_compute_memmove::compute_block_access(
          loop * mem_line_ratio_ * numlines * mem_op_size_);
      doing_memory_ = false;

      eventdata::ptr inst = eventdata::construct(get_lib_name(), timestamp(0));
      inst->set_event(INST_FLOP_LABEL, loop * flop_line_ratio_ * numlines);
      lib_compute_inst::compute_inst(inst);
    }

    void
    lib_compute_loops::compute_loop3(int from1, int to1, int from2, int to2,
        int from3, int to3, int numlines)
    {
      int loop1 = to1 - from1 + 1; //+1 for fortran loops
      int loop2 = to2 - from2 + 1;
      int loop3 = to3 - from3 + 1;
      int loop = loop1 * loop2 * loop3;

      doing_memory_ = true;
      lib_compute_memmove::compute_block_access(
          loop * mem_line_ratio_ * numlines * mem_op_size_);
      doing_memory_ = false;

      eventdata::ptr inst = eventdata::construct(get_lib_name(), timestamp(0));
      inst->set_event(INST_FLOP_LABEL, loop * flop_line_ratio_ * numlines);
      lib_compute_inst::compute_inst(inst);
    }

    void
    lib_compute_loops::compute_loop4(int from1, int to1, int from2, int to2,
        int from3, int to3, int from4, int to4, int numlines)
    {
      int loop1 = to1 - from1 + 1; //+1 for fortran loops
      int loop2 = to2 - from2 + 1;
      int loop3 = to3 - from3 + 1;
      int loop4 = to4 - from4 + 1;
      int loop = loop1 * loop2 * loop3 * loop4;

      doing_memory_ = true;
      lib_compute_memmove::compute_block_access(
          loop * mem_line_ratio_ * numlines * mem_op_size_);
      doing_memory_ = false;

      eventdata::ptr inst = eventdata::construct(get_lib_name(), timestamp(0));
      inst->set_event(INST_FLOP_LABEL, loop * flop_line_ratio_ * numlines);
      lib_compute_inst::compute_inst(inst);
    }

    void
    lib_compute_loops::compute_fft()
    {
      eventdata::ptr inst = eventdata::construct(get_lib_name(), timestamp(0));
      inst->set_event(INST_FLOP_LABEL, 500);
      lib_compute_inst::compute_inst(inst);
    }

  }
} //end of namespace sstmac
