/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/compute/lib_compute_memmove.h>

namespace sstmac
{
  namespace sw
  {

    using namespace std;

    debug lib_compute_memmove::dbg_;

    void
    lib_compute_memmove::compute_block_access(long bytes)
    {
      if (supported())
        {
          sst_message::ptr cmsg = mem_access::construct(bytes, get_lib_name());

          SSTMAC_DEBUG
                << "lib_compute_memmove: executing local mem access kernel with bytes "
                << bytes << "\n";

          os_->execute_kernel(AMI::AMI_COMPUTE, AMI::AMI_COMP_MEMACCESS, cmsg);

          SSTMAC_DEBUG << "lib_compute_memmove: blocking \n";
          wait_this(cmsg);

        }
      else
        {
          throw valueerror(
              "lib_compute_memmove requires the presence of a memory model");
        }
    }

    void
    lib_compute_memmove::compute_block_access(long bytes, long srcthread, long destthread)
    {
      if (supported())
        {
          sst_message::ptr cmsg = mem_access::construct(bytes, get_lib_name(), srcthread, destthread);

          SSTMAC_DEBUG
                << "lib_compute_memmove: executing mem access kernel with bytes "
                << bytes << " at thread " << destthread << "\n";

          os_->execute_kernel(AMI::AMI_COMPUTE, AMI::AMI_COMP_MEMACCESS, cmsg);

          SSTMAC_DEBUG << "lib_compute_memmove: blocking \n";
          wait_this(cmsg);

        }
      else
        {
          throw valueerror(
              "lib_compute_memmove requires the presence of a memory model");
        }
    }

  }
} //end of namespace sstmac
