/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_MEMMOVE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_COMPUTE_LIB_COMPUTE_MEMMOVE_H_INCLUDED

#include <sstmac/software/libraries/compute/lib_compute_inst.h>
#include <sstmac/software/libraries/compute/mem_access.h>

#define BLOCK_SIZE 512

namespace sstmac
{
  namespace sw
  {

    class lib_compute_memmove : virtual public library
    {
      // ------- constructor / boost stuff -------------//

    protected:
      lib_compute_memmove(softwareid id)
      {
        if (!dbg_.is_real())
          {
            dbg_ = debug("<debug> lib_compute_memmove | lib_memory");
            logger::register_logger(&dbg_);
          }

        libname_ = "computelibmemmove" + id.to_string();
      }

      lib_compute_memmove(std::string libname)
      {
        if (!dbg_.is_real())
          {
            dbg_ = debug("<debug> lib_compute_memmove | lib_memory");
            logger::register_logger(&dbg_);
          }

        libname_ = "computelibmemmove" + libname;
      }

    public:
      typedef boost::intrusive_ptr<lib_compute_memmove> ptr;

      virtual
      ~lib_compute_memmove()
      {
      }

      static ptr
      construct(softwareid id)
      {
        return ptr(new lib_compute_memmove(id));
      }

      static ptr
      construct(std::string libname)
      {
        return ptr(new lib_compute_memmove(libname));
      }
      // --------------------------------------//

      virtual bool
      supported()
      {
        return os_->kernel_supported(AMI::AMI_COMPUTE, AMI::AMI_COMP_MEMACCESS);
      }

    public:

      void
      compute_block_access(long bytes);

      void
      compute_block_access(long bytes, long srcthread, long destthread);

    protected:

      static debug dbg_;



    };

  }
} //end of namespace sstmac

#endif
