/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_HARDWARE_DISK_DISK_MESSAGE_H_INCLUDED
#define SSTMAC_HARDWARE_DISK_DISK_MESSAGE_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>

namespace sstmac { namespace sw
{

  class disk_message : public sst_message
  {
  public:
    enum DISK_ACCESS_TYPE {OPEN, READ, WRITE, CLOSE};

  protected:
    disk_message(std::string libn, DISK_ACCESS_TYPE t, long bytes, long sector, long numsectors, long uid);



  public:
    typedef boost::intrusive_ptr<disk_message> ptr;



    static ptr
    construct(std::string libn, DISK_ACCESS_TYPE t, long bytes, long sector, long numsectors, long uid){
      return ptr(new disk_message(libn, t, bytes, sector, numsectors, uid));
    }
    virtual
    ~disk_message() { }

    virtual std::string
    to_string() const {
      return "disk_message";
    }

    DISK_ACCESS_TYPE
    get_access_type(){
      return accesstype_;
    }

    long
    get_byte_length(){
      return bytes_;
    }

    long
    get_sector(){
      return sector_;
    }

    long get_numsectors(){
      return numsectors_;
    }

    long get_uid(){
      return uid_;
    }

  protected:
    DISK_ACCESS_TYPE accesstype_;
    long bytes_;
    long sector_;
    long numsectors_;
    long uid_;
  };

} } /* namespace sstmac */
#endif /* DISK_MESSAGE_H_ */
