/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */
#ifndef SSTMAC_SOFTWARE_LIBRARIES_FILESYSTEM_FILE_METADATA_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_FILESYSTEM_FILE_METADATA_H_INCLUDED

#include <sstmac/common/messages/sst_message.h>

namespace sstmac
{
  namespace sw
  {

    class file_metadata : public sst_message
    {
    public:
      enum FILE_OPS
      {
        CREATE, CREATE_RESP, OPEN, OPEN_RESP, CLOSE, READ, READ_RESP, WRITE, WRITE_RESP
      };

    protected:
      file_metadata(std::string libn, std::string fname, FILE_OPS op) :
            sst_message(
                libn,
                timestamp(0),
                (op == CREATE || op == OPEN || op == CLOSE) ? sst_message::FILEMETA
                    : sst_message::FILEDATA, 0), op_(op), fname_(fname),
            bytes_(0), write_resp_(false)
      {

      }

    public:
      typedef boost::intrusive_ptr<file_metadata> ptr;

      static ptr
      construct(std::string libn, std::string fname, FILE_OPS op)
      {
        return ptr(new file_metadata(libn, fname, op));
      }

      virtual
      ~file_metadata()
      {
      }

      virtual long
      get_byte_length() const
      {
        return bytes_;
      }

      FILE_OPS op_;
      std::string fname_;
      ptr_type::ptr data_;
      long bytes_;
      long access_size_;
      std::string from_lib_;
      bool write_resp_;

      virtual std::string
      to_string() const
      {
        std::stringstream ss;
        ss << "file_metadata( op = " << op_ << ", fname = " << fname_
            << ", bytes = " << bytes_ << ", access_size " << access_size_
            << ", from_lib = " << from_lib_ << ")";
        return ss.str();
      }
    };

  }
}/* namespace sstmac */
#endif /* FILE_METADATA_H_ */
