/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_COMPONENTS_HPX_COMPONENT_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_COMPONENTS_HPX_COMPONENT_H_INCLUDED

#include <sstmac/software/libraries/library.h>
#include <hpx/agas/globalid.h>


namespace hpx
{
  class hpx_thread_manager;

  namespace components
  {

    class hpx_component : public sstmac::ptr_type
    {

    protected:
      globalid gid_;
      static sstmac::debug dbg_;

      sstmac::sw::OS_API::ptr curr_loc_;

      hpx_component(globalid gid) :
          gid_(gid)
      {
        if (!dbg_.is_real())
          {
            dbg_ = sstmac::debug("<debug> hpx | hpx_component",
                sstmac::logger::blue);
            sstmac::logger::register_logger(&dbg_);
          }
      }

    public:
      typedef boost::intrusive_ptr<hpx_component> ptr;

      virtual std::string
      to_string() const
      {
        return "hpx_component";
      }

      virtual ptr
      clone(globalid gid) = 0;

      virtual void
      register_actions(boost::intrusive_ptr<hpx_thread_manager> threadman) = 0;

      virtual void
      set_curr_locality(sstmac::sw::OS_API::ptr os) {
        curr_loc_ = os;
      }

      virtual void
      register_lib(sstmac::sw::library::ptr lib);

      virtual
      ~hpx_component()
      {
      }

    };

  }
}
#endif
