/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_LCOS_LOCAL_MUTEX_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_LCOS_LOCAL_MUTEX_H_INCLUDED

#include <hpx/lcos/lco.h>
#include <hpx/hpx_app.h>

namespace hpx
{
  namespace lcos
  {

    /**
     * Mutex in SST/macro is actually quite simple, since only one thread is ever
     * active at a time, there's never a chance for real resource contention.
     */
    class local_mutex : public lco
    {

    protected:
      bool locked_;
      std::queue<sstmac::sw::key::ptr> waiting_;

      local_mutex() :
          locked_(false)
      {

      }

    public:
      typedef boost::intrusive_ptr<local_mutex> ptr;

      static ptr
      construct(){
        return ptr(new local_mutex());
      }

      virtual
      ~local_mutex() { }

      void
      lock()
      {
        SSTMAC_DEBUG << "hpx_local_mutex: locking, waiting size is "
              << waiting_.size() << "\n";
        if (locked_)
          {
            sstmac::sw::key::ptr blocker = sstmac::sw::key::construct();
            waiting_.push(blocker);
            hpx_app::hpx_block(blocker);
            hpx_app::hpx_thread_swap();
          }

        locked_ = true;
      }

      void
      unlock()
      {
        SSTMAC_DEBUG << "hpx_local_mutex: unlocking, waiting size is "
              << waiting_.size() << "\n";
        locked_ = false;

        if (waiting_.size() > 0)
          {
            sstmac::sw::key::ptr blocked = waiting_.front();
            waiting_.pop();
            hpx_app::hpx_unblock(blocked);

          }
      }

    };

  }
}

#endif
