/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_REMOTE_ACTION_RESULT_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_REMOTE_ACTION_RESULT_H_INCLUDED

#include <hpx/parcels/parcel.h>
#include <hpx/hpx_app.h>
namespace hpx
{

  namespace parcels
  {

    class remote_action_result : public parcel
    {

    public:
      typedef boost::intrusive_ptr<remote_action_result> ptr;

      static ptr
      construct(sstmac::payload::const_ptr res, long trans,
          sstmac::timestamp now)
      {
        if (stats_[REMOTE_ACTION_RESULT] == sstmac::stat_time_window_sum::ptr())
          {
            stats_[REMOTE_ACTION_RESULT] =
                sstmac::stat_time_window_sum::construct(
                    "<stats> hpx_parcels | hpx",
                    "hpx_parcels_remote_action_result.csv");
          }
        stats_[REMOTE_ACTION_RESULT]->record(now, 1);
        return ptr(new remote_action_result(res, trans));
      }

      sstmac::payload::const_ptr
      result() const
      {
        return result_;
      }

      long
      trans_id() const
      {
        return trans_id_;
      }

      virtual
      ~remote_action_result()
      {
      }

      virtual long
      get_byte_length() const
      {
        return 8 + (result_ ? result_->get_byte_length() : 0);
      }

    protected:
      remote_action_result(sstmac::payload::const_ptr res, long trans) :
          parcel(REMOTE_ACTION_RESULT), result_(res), trans_id_(trans)
      {

      }

      sstmac::payload::const_ptr result_;
      long trans_id_;

    };

  }
}
#endif
