/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_RESOLVE_GID_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_PARCELS_RESOLVE_GID_H_INCLUDED

#include <hpx/parcels/parcel.h>
#include <hpx/hpx_app.h>

namespace hpx
{

  namespace parcels
  {

    class resolve_gid : public parcel
    {

    public:
      typedef boost::intrusive_ptr<resolve_gid> ptr;

      static ptr
      construct(globalid gid, sstmac::nodeaddress::ptr ori, long wait, sstmac::timestamp now)
      {
        if (stats_[RESOLVE_GID] == sstmac::stat_time_window_sum::ptr())
          {
            stats_[RESOLVE_GID] = sstmac::stat_time_window_sum::construct(
                "<stats> hpx_parcels | hpx", "hpx_parcels_resolve_gid.csv");
          }
        stats_[RESOLVE_GID]->record(now, 1);

        return ptr(new resolve_gid(gid, ori, wait));
      }

      globalid
      gid() const
      {
        return gid_;
      }

      sstmac::nodeaddress::ptr
      origin() const
      {
        return origin_;
      }

      long
      wait_id() const
      {
        return waiting_;
      }

      virtual long
      get_byte_length() const
      {
        return 16 + 8 + 8;
      }

    protected:
      resolve_gid(globalid gid, sstmac::nodeaddress::ptr ori, long waitid) :
          parcel(RESOLVE_GID), gid_(gid), origin_(ori), waiting_(waitid)
      {

      }

      globalid gid_;
      sstmac::nodeaddress::ptr origin_;
      long waiting_;
    };

  }
}
#endif
