/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_HPX_RUNTIME_ACTIONS_HPX_ACTION_PARAMS_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_HPX_RUNTIME_ACTIONS_HPX_ACTION_PARAMS_H_INCLUDED

#include <sstmac/common/sizeable.h>
namespace hpx
{

  class hpx_action_params : public virtual sstmac::sizeable
  {

  public:
    typedef boost::intrusive_ptr<hpx_action_params> ptr;

    virtual std::string
    to_string() const
    {
      return "hpx_action_params";
    }

    virtual
    ~hpx_action_params()
    {
    }

    virtual long
    get_byte_length() const
    {
      return size_;
    }

  protected:
    long size_;
  };

  template<typename Arg0Type>
    class hpx_action_params1 : public hpx_action_params
    {

    protected:
      Arg0Type arg0_;

      hpx_action_params1(Arg0Type arg0, long arg0size) :
          arg0_(arg0)
      {
        size_ = arg0size;
      }

    public:
      typedef boost::intrusive_ptr<hpx_action_params1> ptr;

      Arg0Type
      get_arg0()
      {
        return arg0_;
      }

      static ptr
      construct(Arg0Type arg0, long arg0size = sizeof(Arg0Type))
      {
        return ptr(new hpx_action_params1(arg0, arg0size));
      }

      virtual
      ~hpx_action_params1()
      {
      }

    };

  template<typename Arg0Type, typename Arg1Type>
    class hpx_action_params2 : public hpx_action_params
    {

    protected:
      Arg0Type arg0_;
      Arg1Type arg1_;

      hpx_action_params2(Arg0Type arg0, Arg1Type arg1, long arg0size, long arg1size) :
          arg0_(arg0), arg1_(arg1)
      {
        size_ = arg0size + arg1size;
      }

    public:
      typedef boost::intrusive_ptr<hpx_action_params2> ptr;

      Arg0Type
      get_arg0()
      {
        return arg0_;
      }

      Arg1Type
      get_arg1()
      {
        return arg1_;
      }

      static ptr
      construct(Arg0Type arg0, Arg1Type arg1, long arg0size = sizeof(Arg0Type), long arg1size = sizeof(Arg1Type))
      {
        return ptr(new hpx_action_params2(arg0, arg1, arg0size, arg1size));
      }

      virtual
      ~hpx_action_params2()
      {
      }



    };

  template<typename Arg0Type, typename Arg1Type, typename Arg2Type>
    class hpx_action_params3 : public hpx_action_params
    {

    protected:
      Arg0Type arg0_;
      Arg1Type arg1_;
      Arg2Type arg2_;

      hpx_action_params3(Arg0Type arg0, Arg1Type arg1, Arg2Type arg2, long arg0size, long arg1size, long arg2size) :
          arg0_(arg0), arg1_(arg1), arg2_(arg2)
      {
        size_ = arg0size + arg1size + arg2size;
      }

    public:
      typedef boost::intrusive_ptr<hpx_action_params3> ptr;

      Arg0Type
      get_arg0()
      {
        return arg0_;
      }

      Arg1Type
      get_arg1()
      {
        return arg1_;
      }

      Arg2Type
      get_arg2()
      {
        return arg2_;
      }

      static ptr
      construct(Arg0Type arg0, Arg1Type arg1, Arg2Type arg2, long arg0size = sizeof(Arg0Type), long arg1size = sizeof(Arg1Type), long arg2size = sizeof(Arg2Type))
      {
        return ptr(new hpx_action_params3(arg0, arg1, arg2, arg0size, arg1size, arg2size));
      }

      virtual
      ~hpx_action_params3()
      {
      }



    };

  template<typename Arg0Type, typename Arg1Type, typename Arg2Type,
      typename Arg3Type>
    class hpx_action_params4 : public hpx_action_params
    {

    protected:
      Arg0Type arg0_;
      Arg1Type arg1_;
      Arg2Type arg2_;
      Arg3Type arg3_;

      hpx_action_params4(Arg0Type arg0, Arg1Type arg1, Arg2Type arg2,
          Arg3Type arg3, long arg0size, long arg1size, long arg2size, long arg3size) :
          arg0_(arg0), arg1_(arg1), arg2_(arg2), arg3_(arg3)
      {
        size_ = arg0size + arg1size + arg2size + arg3size;
      }

    public:
      typedef boost::intrusive_ptr<hpx_action_params4> ptr;

      Arg0Type
      get_arg0()
      {
        return arg0_;
      }

      Arg1Type
      get_arg1()
      {
        return arg1_;
      }

      Arg2Type
      get_arg2()
      {
        return arg2_;
      }

      Arg3Type
      get_arg3()
      {
        return arg3_;
      }

      static ptr
      construct(Arg0Type arg0, Arg1Type arg1, Arg2Type arg2, Arg3Type arg3, long arg0size = sizeof(Arg0Type), long arg1size = sizeof(Arg1Type), long arg2size = sizeof(Arg2Type), long arg3size = sizeof(Arg3Type))
      {
        return ptr(new hpx_action_params4(arg0, arg1, arg2, arg3, arg0size, arg1size, arg2size, arg3size));
      }

      virtual
      ~hpx_action_params4()
      {
      }



    };

}

#endif
