/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_comm/mpicommfactory-pending-keylist.h>
#include <sstmac/common/errors.h>

namespace sstmac { namespace sw {

  //
  // Hello.
  //
  mpicommfactory::pending_split::keylist::keylist(const mpicommid &id) :
    id_(id)
  {}

  //
  // Goodbye.
  //
  mpicommfactory::pending_split::keylist::~keylist() {
  }

  //
  // Construct a new keylist.
  //
  mpicommfactory::pending_split::keylist::ptr
  mpicommfactory::pending_split::keylist::construct(const mpicommid &id)
  {
    return keylist::ptr(new keylist(id));
  }

  //
  // Return true if no ranks have been added yet.
  //
  bool mpicommfactory::pending_split::keylist::empty() const {
    return ranks_.empty();
  }

  //
  // Return true if shared state has been constructed.
  //

  bool mpicommfactory::pending_split::keylist::locked() const {
    return (shared_ != mpicommdata::ptr());
  }

  //
  // Test whether the given rank is already defined in the keylist.
  //
  bool mpicommfactory::pending_split::keylist::contains(int key) const {
    return (key < 0 || (ranks_.find(key) != ranks_.end()));
  }

  //
  // Insert a new rank in the keylist.
  //
  void mpicommfactory::pending_split::keylist::
  insert(int key, const taskid &tid, nodeaddress::ptr &nid
      )
  {
    //std::cerr << "mpicommfactory::pending_split::keylist[" << this
    //          << "]::insert(" << key << ", " << tid << ", " << nid << ")\n";
    if(this->locked())
      throw illformederror("mpicommfactory::pending_split::keylist::insert:  "
                           "Keylist is locked -- no more insertions.");
    if(key >= 0) {
      if(this->contains(key))
        throw valueerror("mpicommfactory::pending_split::keylist::insert:  "
                         "Attempt to re-define an existing key.");
      //std::cerr << "keylist:  Setting rank for key " << key << "\n";
      ////ranks_[key] = location(tid, nid);
      //std::cerr << "Total ranks are now " << ranks_.size() << "\n";
    }
  }

  //
  // Get a communicator corresponding to the given key.
  //
  mpicomm mpicommfactory::pending_split::keylist::get_mpicomm(int key) {
    //std::cerr << "mpicommfactory::pending_split::keylist[" << this
    //          << "]::get_mpicomm(" << key << ")\n";
   /* if(! this->contains(key))
      throw illformederror("mpicommfactory::pending_split::keylist::get_mpicomm"
                           ":  Key is not defined in the keylist.");
    if(! shared_) {
      std::vector<taskid> peers;
      ////std::vector<nodeid> nodes;
      peers.reserve(ranks_.size());
    ////  nodes.reserve(ranks_.size());
      rankmap_t::iterator it, end = ranks_.end();
      mpiid currid(0);
      for(it = ranks_.begin(); it != end; ++it) {
        peers.push_back(it->second.the_task);
       //// nodes.push_back(it->second.the_node);
        it->second.final_rank = currid;
        currid.id++;
      }
      shared_ = mpicommdata::construct(id_, peers////, nodes
          );
      //std::cerr << "mpicommfactory::pending_split::keylist::get_mpicomm: Commid " << id_ << " points to data object " << shared_ << "\n";
    }
    //std::cerr << "mpicommfactory::pending_split::keylist::get_mpicomm:  "
    //          << "Return reached for key " << key << "\n";
    if(key >= 0) {
      return mpicomm(ranks_[key].final_rank, shared_);
    }
    else {
      return mpicomm::comm_null;
    }*/

    throw unimplementederror("mpicommfactory::pending_split::keylist::get_mpicomm not implemented");
  }

} } // end of namespace sstmac
