/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_queue/mpiqueue-recvrequest.h>

#include <sstmac/software/libraries/mpi/mpimessage.h>
#include <sstmac/common/debug.h>
#include <sstmac/common/errors.h>

namespace sstmac { namespace sw
{

  // Pointer types.
  
  

  //
  // Hello.
  //
  mpiqueue::recvrequest::recvrequest(const mpirequest_t key,
      const mpiqueue::ptr& queue, int count, const mpitype &type,
      const mpiid &source, const mpitag &tag, const mpicomm &comm,
      mpimessage::category cat, eventhandler::ptr &completion) :
    queue_(queue), acked_(false), source_(source), tag_(tag), comm_(comm),
        cat_(cat), count_(count), type_(type), key_(key),
        completion_(completion)
  {
  }



  //
  // Hello.
  //
 mpiqueue::recvrequest::ptr
  mpiqueue::recvrequest::construct(const mpirequest_t key,
      const mpiqueue::ptr& queue, int count, const mpitype &type,
      const mpiid &source, const mpitag &tag, const mpicomm &comm,
      mpimessage::category cat, eventhandler::ptr &completion)
  {
    recvrequest::ptr retval(
        new recvrequest(key, queue, count, type, source, tag, comm, cat,
            completion));

    return retval;
  }

  //
  // Goodbye.
  //
  mpiqueue::recvrequest::~recvrequest()
  {
   // std::cout << "calling recvrequest destructor" << std::endl;
  }

  //
  // Do we match the given message?
  //
  //   bool mpiqueue::recvrequest::matches(mpimessage::ptr msg) const {
  //     bool fit = ((cat_ == msg->cat()) && (comm_.id() == msg->commid()) &&
  //                 source_.matches(msg->source()) && tag_.matches(msg->tag()));
  //     if(fit) {
  //       // Just a little error checking.
  //       if(type_ != msg->type() || count_ != abs(msg->count())) {
  //         if(debugger::active()) debug() << "WARNING:  mpiqueue::recvrequest::match:  Matching message "
  //                 << "but not matching type and count.  Expected " << count_
  //                 << " elements of type " << type_ << " but received "
  //                 << msg->count() << " elements of type " << msg->type() << "\n";
  //       }
  //     }
  //     return fit;
  //   }

  //
  // We be done.
  //
  void
  mpiqueue::recvrequest::handle(const mpimessage::ptr& mess)
  {
    // Figure out whether this is a full message or just a handshake.
    if (mess->count() < 0)
      {
        // Ack that we received the handshake.
        queue_->send_ack(mess, recvrequest::ptr(this));
        acked_ = true;
        // We're done for now -- the queue will deliver the message later.
      }
    else
      {
        // We get here if this was a ready-send message or if
        // we already acknowledged the handshake and the message just arrived.
        if (mess->ssend())
          {
            if (!acked_)
              {
                // We can ack, but we will not be receiving a response.
                queue_->send_ack(mess, recvrequest::ptr ());
              }
          }
        key_->complete(mess);
        completion_->handle(mess);

      }

  }

} } // end of namespace sstmac
