/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_strategies/mpicoreallgather.h>

#include <sstmac/software/libraries/unblockhandler.h>
#include <sstmac/software/libraries/mpi/mpi_strategies/mpi_collective/engines/mpiringallgather.h>

namespace sstmac { namespace sw
{

  // Pointer types.
  

  //
  // Hi.
  //
  mpicoreallgather::mpicoreallgather()
  {
  }

  //
  // Hello.
  //
  mpicoreallgather::ptr
  mpicoreallgather::construct()
  {
    return mpicoreallgather::ptr (new mpicoreallgather);
  }

  //
  // Goodbye.
  //
  mpicoreallgather::~mpicoreallgather() throw ()
  {
  }

  //
  // Build a kernel for an allgather operation
  //
  void
  mpicoreallgather::execute(const mpirequest_t& thekey, const
      mpiqueue::ptr& queue, int sendcount, const mpitype &sendtype, int recvcount,
      const mpitype &recvtype, const mpitag &tag, const mpicomm &comm,
      const payload::const_ptr& content, OS_API::ptr &os) const
  {
    // return mpiringallgather::construct(thekey, queue, sendcount, sendtype,
    //                                   recvcount, recvtype, tag, comm,
    //                                   content, magic);

    key::ptr k = thekey->get_key();
    eventhandler::ptr completion = unblockhandler::construct(os, k);
    mpiringallgather::ptr engine = mpiringallgather::construct(
        thekey, queue, sendcount, sendtype, recvcount, recvtype, tag,
        comm, content, completion);

    engine->start();
  }

} } // end of namespace sstmac
