/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_strategies/mpicoreallreduce.h>

#include <sstmac/software/libraries/mpi/mpi_strategies/mpi_collective/engines/mpirabenseifner.h>
#include <sstmac/software/libraries/unblockhandler.h>

namespace sstmac { namespace sw
{

  // Pointer types.
  

  //
  // Hi.
  //
  mpicoreallreduce::mpicoreallreduce()
  {
  }

  //
  // Hello.
  //
  mpicoreallreduce::ptr
  mpicoreallreduce::construct()
  {
    return mpicoreallreduce::ptr (new mpicoreallreduce);
  }

  //
  // Perform a fused allreduce-allreduce operation
  //
  void
  mpicoreallreduce::execute(const mpirequest_t& thekey, const
      mpiqueue::ptr& queue, int count, const mpitype &type, const mpiop &op,
      const mpitag &tag, const mpicomm &comm,
      const payload::const_ptr& content, OS_API::ptr &os) const
  {

    key::ptr k = thekey->get_key();
    eventhandler::ptr completion = unblockhandler::construct(os, k);
    mpirabenseifner::ptr engine = mpirabenseifner::construct(
        true, thekey, queue, count, type, op, mpiid(-1), tag, comm, content,
        completion);

    engine->start();
  }

} } // end of namespace sstmac
