/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/libraries/mpi/mpi_strategies/mpicorersend.h>

#include <sstmac/software/libraries/mpi/mpi_queue/mpiqueue.h>
#include <sstmac/software/libraries/unblockhandler.h>

namespace sstmac { namespace sw
{

  
  

  //
  // Hello.
  //
  mpicorersend::mpicorersend()
  {
  }

  //
  // Build.
  //
  mpicorersend::ptr
  mpicorersend::construct()
  {
    return mpicorersend::ptr (new mpicorersend);
  }

  //
  // Goodbye.
  //
  mpicorersend::~mpicorersend() throw ()
  {
  }

  //
  // Basic nonblocking send.
  //
  void
  mpicorersend::execute(const mpirequest_t& thekey, const
      mpiqueue::ptr& queue, int count, const mpitype &type, const mpiid &dest,
      const mpitag &tag, const mpicomm &comm,
          const payload::const_ptr& content, OS_API::ptr &os) const
  {
    bool rsend = true, ssend = false;
    key::ptr k = thekey->get_key();
    eventhandler::ptr completion = unblockhandler::construct(os,
        k);
    queue->send(thekey, count, type, dest, tag, comm, rsend, ssend,
        mpimessage::user, completion, content);
  }

} } // end of namespace sstmac.
