!
!  This file is part of SST/macroscale: 
!               The macroscale architecture simulator from the SST suite.
!  Copyright (c) 2009-2011 Sandia Corporation.
!  This software is distributed under the BSD License.
!  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
!  the U.S. Government retains certain rights in this software.
!  For more information, see the LICENSE file in the top 
!  SST/macroscale directory.
!

!module MPI
! These are hard-mapped to sstmac_mpi.h
  integer :: MPI_INTEGER = 5, MPI_REAL = 6, MPI_DOUBLE_PRECISION = 7 
  integer, parameter :: MPI_STATUS_SIZE = 4
!end module MPI_TYPES

integer, parameter :: MPI_MAX = 1, MPI_MIN = 2, MPI_SUM = 3, MPI_PROD = 4, MPI_LAND = 5, MPI_BAND = 6
integer, parameter :: MPI_LOR = 7, MPI_BOR = 8, MPI_LXOR = 9, MPI_BXOR = 10, MPI_MINLOC = 11, MPI_MAXLOC = 12


INTERFACE 
   SUBROUTINE MPI_Init(err)
     INTEGER, INTENT(OUT) :: err
   END SUBROUTINE MPI_Init
END INTERFACE

INTERFACE
   SUBROUTINE MPI_Finalize(err)
     INTEGER, INTENT(OUT) :: err
   END SUBROUTINE MPI_Finalize
END INTERFACE

interface
	subroutine MPI_Comm_World(comm)
		integer :: comm
	end subroutine MPI_Comm_World
end interface

interface
	subroutine MPI_Abort(comm, code, err)
		integer :: comm, code, err
	end subroutine MPI_Abort
end interface

interface
	subroutine MPI_Comm_Rank(comm, rank, err)
		integer :: comm, rank, err
	end subroutine MPI_Comm_Rank
end interface

interface
	subroutine MPI_Comm_Size(comm, size, err)
		integer :: comm, size, err
	end subroutine MPI_Comm_Size
end interface

interface MPI_Send
  subroutine MPI_Send0(cnt, datatype, dest, tag, comm, err)
	integer :: cnt, dest, tag, comm, err
	integer :: datatype
  end subroutine MPI_Send0

  subroutine MPI_Send1(buf, cnt, datatype, dest, tag, comm, err)
	integer :: buf
	integer :: cnt, dest, tag, comm, err
	integer :: datatype
  end subroutine MPI_Send1

!  subroutine MPI_Send2(buf, cnt, datatype, dest, tag, comm, err)
!	integer :: cnt, dest, tag, comm, err
!	integer, dimension(cnt) :: buf
!	integer :: datatype
!  end subroutine MPI_Send2

 subroutine MPI_Send3(buf, cnt, datatype, dest, tag, comm, err)
	real :: buf
	integer :: cnt, dest, tag, comm, err
	integer :: datatype
  end subroutine MPI_Send3

end interface MPI_Send

interface MPI_Recv
 subroutine MPI_Recv0(cnt, datatype, src, tag, comm, status, err)
	integer :: cnt, src, tag, comm, err
	integer :: datatype
	integer, dimension(:) :: status
  end subroutine MPI_Recv0

  subroutine MPI_Recv1(buf, cnt, datatype, src, tag, comm, status, err)
	integer :: buf
	integer :: cnt, src, tag, comm, err
	integer :: datatype
	integer, dimension(:) :: status
  end subroutine MPI_Recv1

!  subroutine MPI_Recv2(buf, cnt, datatype, src, tag, comm, status, err)
!	integer :: cnt, src, tag, comm, err
!	integer, dimension(cnt) :: buf
!	integer :: datatype
!	integer, dimension(:) :: status
! end subroutine MPI_Recv2

subroutine MPI_Recv3(buf, cnt, datatype, src, tag, comm, status, err)
	real :: buf
	integer :: cnt, src, tag, comm, err
	integer :: datatype
	integer, dimension(:) :: status
  end subroutine MPI_Recv3

end interface MPI_Recv

INTERFACE
	SUBROUTINE MPI_Wait(request, status, err)
	  integer, INTENT(IN) :: request
	  integer, dimension(:) :: status
	  INTEGER, INTENT(OUT) :: err
	END SUBROUTINE MPI_Wait
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Waitany(cnt, requests, idx, status, err)
	  integer :: cnt, idx
	  integer, dimension(:) :: requests, status
	  INTEGER, INTENT(OUT) :: err
	END SUBROUTINE MPI_Waitany
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Waitsome(incnt, requests, outcnt, indices, statuses, err)
	  integer :: incnt, outcnt, err
	  integer, dimension(:) :: requests, indices
	  integer, dimension(:,:) :: statuses
	END SUBROUTINE MPI_Waitsome
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Get_count(status, datatype, cnt, err)
	  integer :: cnt, err, datatype
	  integer, dimension(:) :: status
	END SUBROUTINE MPI_Get_count
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Test(request, flag, status, err)
	  integer :: request, flag, err
	  integer, dimension(:) :: status
	END SUBROUTINE MPI_Test
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Cancel(request, err)
	  integer :: request, err
	END SUBROUTINE MPI_Cancel
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Request_free(request, err)
	  integer :: request, err
	END SUBROUTINE MPI_Request_free
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Comm_dup(comm, newcomm, err)
	  integer :: comm, newcomm, err
	END SUBROUTINE MPI_Comm_dup
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Comm_split(comm, color, key, newcomm, err)
	  integer :: comm, color, key, newcomm, err
	END SUBROUTINE MPI_Comm_split
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Comm_free(comm, err)
	  integer :: comm, err
	END SUBROUTINE MPI_Comm_free
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Comm_create(comm, group, newcomm, err)
	  integer :: comm, group, newcomm, err
	END SUBROUTINE MPI_Comm_create
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Comm_group(comm, group, err)
	  integer :: comm, group, err
	END SUBROUTINE MPI_Comm_group
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Group_incl(group, n, ranks, newgroup, err)
	  integer :: group, n, newgroup, err
	  integer, dimension(:) :: ranks
	END SUBROUTINE MPI_Group_incl
END INTERFACE

INTERFACE
	SUBROUTINE MPI_Group_free(group, err)
	  integer :: group, err
	END SUBROUTINE MPI_Group_free
END INTERFACE

interface MPI_SendRecv
  subroutine MPI_SendRecv0(sendcnt, senddatatype, dest, sendtag, recvcount, recvtype, source, recvtag, comm, status, err)
	integer :: sendcnt, senddatatype, dest, sendtag, recvcount, recvtype, source, recvtag, comm, err
	integer, dimension(:) :: status
  end subroutine MPI_SendRecv0

end interface MPI_SendRecv

interface MPI_Isend
  subroutine MPI_Isend0(cnt, datatype, dest, tag, comm, request, err)
	integer :: cnt, dest, tag, comm, err, request
	integer :: datatype
  end subroutine MPI_Isend0

end interface MPI_Isend

interface MPI_Irecv
  subroutine MPI_Irecv0(cnt, datatype, source, tag, comm, request, err)
	integer :: cnt, source, tag, comm, err, request
	integer :: datatype
  end subroutine MPI_Irecv0

end interface MPI_Irecv


INTERFACE
	SUBROUTINE MPI_Recover_payload(buf, request, err)
	  integer :: buf, request, err
	END SUBROUTINE MPI_Recover_payload
END INTERFACE


interface MPI_Allreduce
  subroutine MPI_Allreduce0(cnt, datatype, op, comm, err)
	integer :: cnt, datatype, op, comm, err
  end subroutine MPI_Allreduce0

end interface MPI_Allreduce

interface MPI_Reduce
  subroutine MPI_Reduce0(cnt, datatype, op, root, comm, err)
	integer :: cnt, datatype, op, root, comm, err
  end subroutine MPI_Reduce0

end interface MPI_Reduce

INTERFACE
	SUBROUTINE MPI_Barrier(comm, err)
	  integer :: comm, err
	END SUBROUTINE MPI_Barrier
END INTERFACE


INTERFACE MPI_Bcast
	SUBROUTINE MPI_Bcast0(cnt, datatype, root, comm, err)
	  integer :: cnt, datatype, root, comm, err
	END SUBROUTINE MPI_Bcast0
END INTERFACE MPI_Bcast

INTERFACE MPI_Gather
	SUBROUTINE MPI_Gather0(sendcnt, sendtype, recvcnt, recvtype, root, comm, err)
	  integer :: sendcnt, sendtype, recvcnt, recvtype, root, comm, err
	END SUBROUTINE MPI_Gather0
END INTERFACE MPI_Gather

INTERFACE MPI_Allgather
	SUBROUTINE MPI_Allgather0(sendcnt, sendtype, recvcnt, recvtype, comm, err)
	  integer :: sendcnt, sendtype, recvcnt, recvtype, comm, err
	END SUBROUTINE MPI_Allgather0
END INTERFACE MPI_Allgather

INTERFACE MPI_Scatter
	SUBROUTINE MPI_Scatter(sendcnt, sendtype, recvcnt, recvtype, root, comm, err)
	  integer :: sendcnt, sendtype, recvcnt, recvtype, root, comm, err
	END SUBROUTINE MPI_Scatter
END INTERFACE MPI_Scatter

INTERFACE MPI_Alltoall
	SUBROUTINE MPI_Alltoall0(sendcnt, sendtype, recvcnt, recvtype,  comm, err)
	  integer :: sendcnt, sendtype, recvcnt, recvtype,  comm, err
	END SUBROUTINE MPI_Alltoall0
END INTERFACE MPI_Alltoall


interface
	subroutine MPI_Wtime(t)
		real :: t
	end subroutine MPI_Wtime
end interface
