/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_MULTICOMPLETE_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_MULTICOMPLETE_H_INCLUDED

#include <sstmac/common/timestamp.h>
#include <sstmac/common/nocopy.h>
#include <sstmac/software/process/key.h>

#include <vector>
#include <map>

namespace sstmac { namespace sw
{

  // Important types.


  /**
   * An eventhandler that monitors the completion status of a collection
   * of requests.
   */
 /* class multicomplete : virtual public nocopy
  {
    /// Private eventhandler type that we use to manage process completions.
    class handler;


    /// The key we use to turn our request on and off.
    key::ptr key_;
    /// The scheduler that handles context switching.
    boost::shared_ptr<scheduler> scheduler_;
    /// Our list of outstanding event handlers.  Gets populated inside
    /// complete_* methods and de-populated upon their return.
    typedef std::map<handler::ptr, boost::shared_ptr<
        processrequest> > handlermap_t;
    handlermap_t handlers_;
    /// The expected number of completions before we're "done".
    int64_t required_completions_;
    /// The index of the last completion to arrive.
    int64_t last_completion_;
    /// The time of the last completion.
    timestamp completion_time_;

    /// Private method used by a handler upon completion.
    void
    handler_done(const timestamp &current_time,
        const handler::ptr& the_handler, int64_t index)
    {
    }

  protected:
    /// Hello.
    multicomplete(const boost::shared_ptr<scheduler>& sched,
        const key::ptr& thekey);



  public:
    typedef boost::intrusive_ptr<multicomplete> ptr;

    /// Hello.
    static multicomplete::ptr
    construct(
        const key::ptr& thekey);

    /// Goodbye.
    virtual
    ~multicomplete() throw ();

    /// Wait for all the given requests to complete.
    /// Marks each request as inactive upon completion.
    /// \param start_time is the return value if all the requests are complete.
    /// \param req is the list of requests.
    /// \return the completion time.
    timestamp
    complete_all(const timestamp &start_time, std::vector<boost::shared_ptr<
        processrequest> > &req);

    /// Wait for one of the given requests to complete.
    /// Marks the completed request as inactive upon completion, but does not
    /// change the activity status of other completed requests.
    /// \param start_time is the return time if at least one of the requests
    ///        is already complete.
    /// \param req is the list of requests.
    /// \param index is the index of the completed requests.  This value
    ///        is less than zero if all the input requests were already
    ///        marked inactive.
    timestamp
    complete_one(const timestamp &start_time, std::vector<boost::shared_ptr<
        processrequest> > &req, int &index);

    /// Wait for one or more of the given requests to complete.
    /// If at least one of the requests is complete at call time, all the
    /// completed requests will be marked as completed.  Otherwise, this
    /// call will have the same effect as complete_one.
    /// \param start_time is the return time if at least one of the requests
    ///        is already complete or if all requests are inactive.
    /// \param req is the list of requests.
    /// \param indices are the indices (displacements) of the requeusts
    ///        that were completed in this call.  This vector is empty if
    ///        no request can be completed (if all are inactive).
    timestamp
    complete_some(const timestamp &start_time, std::vector<boost::shared_ptr<
        processrequest> > &req, std::vector<int> &indices);
  };*/

} } // end of namespace sstmac

#endif
