/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_LIBRARIES_UTIL_LIB_DISPLAY_H_INCLUDED
#define SSTMAC_SOFTWARE_LIBRARIES_UTIL_LIB_DISPLAY_H_INCLUDED

#include <sstmac/software/libraries/library.h>
#include <sstmac/software/process/softwareid.h>
#include <sstmac/common/sstmac-config.h>

#include <map>

namespace sstmac { namespace sw
{

  enum DISPLAY_COLORS
  {
    BLACK = 0, RED = 1, GREEN = 2, YELLOW = 3, ORANGE = 4, PURPLE = 5, PINK = 6, GREY = 7

  };

  class lib_display : public library
  {
    // ------- constructor / boost stuff -------------//


  protected:
    lib_display(softwareid id){

      libname_ = "displaylib" + id.to_string();
    }


  public:
    typedef boost::intrusive_ptr<lib_display> ptr;

    virtual
    ~lib_display() { }

    static ptr
    construct(softwareid id){
      return ptr(new lib_display(id));
    }
    // --------------------------------------//

  public:

    void
    change_border(DISPLAY_COLORS c);

    void
    change_fill(DISPLAY_COLORS c);


  };

} } //end of namespace sstmac

#endif
