/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMACRO_APPDESCR_H
#define SSTMACRO_APPDESCR_H

#include <sstmac/software/process/app.h>
#include <sstmac/common/sim_parameters.h>

#define SSTRegisterApp(appstr, appname) static sstmac::sw::SSTAppDesc<appname> appname##_cd(appstr);

namespace sstmac {
    namespace sw {

class SSTAppDesc_base 
{

    public:
        typedef boost::intrusive_ptr<SSTAppDesc_base> ptr;

        virtual app::ptr build_app() = 0;

};

class SSTAppDesc_build {
    private:
        static std::map<const char*, SSTAppDesc_base*>* appmap_;

    public:
        static void register_app(const char* name, SSTAppDesc_base* descr);

        static void init_apps(sim_parameters::ptr);

};

template <class T>
class SSTAppDesc :
    public SSTAppDesc_base
{

    public:
        SSTAppDesc(const char* name)
        {
            /** This should only ever be declared within 
                the static initialization routine. There is therefore
                nothing unsafe about passing "this" out.  This object
                will be persisent throughout the entire simulation */
            SSTAppDesc_build::register_app(name, this); 
        }

        app::ptr build_app()
        {
            appid aid(0);
            taskid tid(0);
            softwareid sid(aid, tid);
            app::ptr application(T::construct(sid));
            return application;
        }
        
};

  } //end namespace sstmac
} //end namespace sw

#endif

