/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_KEY_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_KEY_H_INCLUDED

//#include <sstmac/software/threading/context.h>
#include <sstmac/software/threading/threading_interface.h>
#include <sstmac/software/process/thread.h>
#include <sstmac/common/ptr_type.h>

#include <vector>
#include <cstring>
#include <set>

namespace sstmac { namespace sw {

  /**
   * A base type and default (empty) implementation of a handle
   * to block and unblock processes.
   */
  class key : public ptr_type {

    friend class operatingsystem;

    // thread_data_t defined in process/thread.h
    typedef std::set<thread_data_t> blocking_t;
    blocking_t blocking_;

  public:

    virtual std::string to_string() const{
            return "key";
          }


    typedef boost::intrusive_ptr<key> ptr;

    static key::ptr construct();

    virtual ~key();

    void* operator new(size_t size);

    void operator delete(void* ptr);

    void
    add_blocker(thread_data_t t); 

    void
    remove_blocker(thread_data_t t);

    bool
    still_blocked(){
      return blocking_.size() > 0;
    }

    void clear(){
      blocking_.clear();
    }
  };

} } // end of namespace sstmac

#endif
