/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_OS_API_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_OS_API_H_INCLUDED

#include <sstmac/software/process/softwareid.h>
#include <sstmac/common/nodeaddress.h>
//#include <sstmac/software/threading/context.h>
#include <sstmac/software/threading/threading_interface.h>
#include <sstmac/common/messages/sst_message.h>
#include <sstmac/software/ami/ami.h>
//#include <sstmac/software/process/thread.h>

#include <sstmac/common/sstmac-config.h>
#include <sstmac/common/ptr_type.h>

#include <sstmac/software/process/key.h>
//#include <sstmac/software/process/thread.h>
//#include <sstmac/software/libraries/library.h>
#include <sstmac/software/ami/environment.h>

namespace sstmac { namespace sw
{

class thread;
class library;
class service;

  class OS_API : public ptr_type
  {
  protected:
    static thread::ptr current_thread_;

  public:

    virtual std::string to_string() const{
            return "os_api";
          }


    typedef boost::intrusive_ptr<OS_API> ptr;

    // -- provides access to AMI --
    // this is done so that AMI's can be swapped out underneath an app at runtime (thread migration)
    virtual void execute_kernel(AMI::AMI_CAT cat, int kernelName, sst_message::ptr data) = 0;
    virtual ptr_type::ptr execute_kernel(AMI::AMI_CAT cat, int kernelName) = 0;
    virtual bool kernel_supported(AMI::AMI_CAT cat, int kernelName) = 0;
    // -----

    // -- threading functions ---
    virtual timestamp block(const key::ptr &req) = 0;
    virtual timestamp unblock(const key::ptr &req) = 0;
    virtual void start_thread(const boost::intrusive_ptr<thread> &t) = 0;
   // virtual void join_thread(const thread::ptr &t, threadcontext_t con) = 0;
    virtual void complete_thread() = 0;
    // -----


    // -- so the OS knows about a lib --
    virtual void register_lib(const boost::intrusive_ptr<library> &lib) = 0;
    virtual void register_ser(const boost::intrusive_ptr<service> &ser) = 0;
    virtual void unregister_lib(const boost::intrusive_ptr<library> &lib) = 0;
    virtual boost::intrusive_ptr<library> get_lib(std::string name) = 0;

    // --------

    // ---- other usefuls ---
    virtual nodeaddress::ptr get_my_addr() = 0;
    virtual nodeaddress::ptr get_task_addr(softwareid id) = 0;
    virtual environment::const_ptr get_env(appid a) = 0;
    // --------

    // --- simulation functions (though apps shouldn't be too aware they're being simulated)
    virtual timestamp now() = 0;
    virtual thread_data_t get_current_context() = 0;

    static boost::intrusive_ptr<thread>
    get_current_thread()
    {
      return current_thread_;
    }

  };

} } //end of namespace sstmac

#endif
