/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_PROCESS_TASKLIST_H_INCLUDED
#define SSTMAC_SOFTWARE_PROCESS_TASKLIST_H_INCLUDED

#include <sstmac/software/process/taskid.h>
#include <sstmac/common/errors.h>
#include <sstmac/common/ptr_type.h>

#include <vector>

namespace sstmac
{
  namespace sw
  {

    class tasklist : public ptr_type
    {

    protected:
      tasklist(std::vector<taskid> &v) :
          list_(v)
      {

      }

      std::vector<taskid> list_;

    public:
      typedef boost::intrusive_ptr<tasklist> ptr;
      typedef boost::intrusive_ptr<const tasklist> const_ptr;

      virtual std::string
      to_string() const
      {
        return "tasklist";
      }

      static ptr
      construct(std::vector<taskid> &v)
      {
        return ptr(new tasklist(v));
      }

      taskid
      at(int i) const
      {
        if (i > list_.size())
          throw rangeerror("value out of range at tasklist");
        else
          return list_[i];
      }

      int
      size() const {
        return list_.size();
      }

      bool
      equals(const tasklist::ptr &other) const
      {
        bool same = list_.size() == other->list_.size();
        for (int i = 0; i < list_.size() && same; i++)
          {
          same = same && (list_[i] == other->list_[i]);
        }
      return same;
    }

    bool
    equals(const std::vector<taskid> &other) const
    {
      bool same = list_.size() == other.size();
      for (int i = 0; i < list_.size() && same; i++)
        {
        same = same && (list_[i] == other[i]);
      }
    return same;
  }

  };

}
} //end of namespace sstmac

#endif
