/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SERVICES_LAUNCH_ALLOCATION_CARTALLOCATION_H_INCLUDED
#define SSTMAC_SOFTWARE_SERVICES_LAUNCH_ALLOCATION_CARTALLOCATION_H_INCLUDED

#include <sstmac/hardware/network/topology/hdtorus.h>
#include <sstmac/software/services/launch/allocation/allocationstrategy.h>

#include <sstmac/common/errors.h>

namespace sstmac
{
  namespace sw
  {

    class cartallocation : public allocationstrategy
    {
      sstmac::hw::hdtorus::ptr network_;
      long xoff_, yoff_, xsize_, ysize_;
      bool relocated_;
      long xorig_, yorig_, xnew_, ynew_;

      cartallocation(long num_avail, const sstmac::hw::hdtorus::ptr& network,
          long xoff, long yoff, long xsize, long ysize, bool relocated = false,
          long xorig = -1, long yorig = -1, long xnew = -1, long ynew = -1);
    public:
      virtual
      ~cartallocation() throw ()
      {
      }
      typedef boost::intrusive_ptr<cartallocation> ptr;
      static ptr
      construct(long num_avail, const sstmac::hw::hdtorus::ptr& network,
          long xoff, long yoff, long xsize, long ysize);
      static ptr
      construct_relocate_node(long num_avail,
          const sstmac::hw::hdtorus::ptr& network, long xoff, long yoff,
          long xsize, long ysize, long xorig, long yorig, long xnew, long ynew);

      static ptr
      construct(sim_parameters::ptr params, const sstmac::hw::topology::ptr& network)
      {
        sstmac::hw::hdtorus::ptr casted = boost::dynamic_pointer_cast<sstmac::hw::hdtorus>(network);
        if(!casted){
            throw valueerror("cartallocation::construct - error in casting the topology to an hdtorus");
        }
        return construct(params->get_long_param("node_num"), casted,
            params->get_long_param("launch_xoff"), params->get_long_param("launch_yoff"),
            params->get_long_param("launch_xsize"), params->get_long_param("launch_ysize"));
      }
      void
      allocate(int nnode,
          std::set<nodeaddress::ptr, nodeaddress::nodeaddresscomp> &allocation);
    };

  }
}

#endif
