/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/services/launch/allocation/rackallocator.h>

namespace sstmac
{
  namespace sw
  {

    using namespace std;

    rackallocator::rackallocator(int numOfNodes, int coresPerProc,
        int procsPerNode, int nodesPerRack, int numRacks) :
        allocationstrategy(numOfNodes), cpp_(coresPerProc), ppn_(procsPerNode), npr_(
            nodesPerRack), nr_(numRacks)
    {

      currentRack = 0;

    }

    void
    rackallocator::allocate(int nnode_requested,
        std::set<nodeaddress::ptr, nodeaddress::nodeaddresscomp> &allocation)
    {
    if(nnode_requested < 0){
    	nnode_requested = available_.size();
    }
      SSTMAC_DEBUG << "numRacks: " << nr_ << ", currentRack: " << currentRack << "\n";
      for(; currentRack < nr_ && allocation.size() < nnode_requested; currentRack++){
          for(long i = 0; i < npr_ && allocation.size() < nnode_requested; i++){
              long id = currentRack * npr_ * cpp_ * ppn_ + i * cpp_ * ppn_;

              nodeaddress::ptr addr = nodeaddress::convert_from_uniqueid(id);
              SSTMAC_DEBUG << "rackallocator: adding uid: " << id << ", addr: " << addr->to_string() << "\n";
              allocated_.insert(addr);
              available_.erase(addr);
              allocation.insert(addr);
          }
      }


    }

  }
}
