/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#include <sstmac/software/services/launch/indexing/blockindexing.h>
#include <sstmac/common/errors.h>

namespace sstmac  { namespace sw
  {

  

  //
  // Default constructor.
  //
  blockindexing::blockindexing() {
  }
  
  blockindexing::~blockindexing() throw() {
  }

  //
  // Static method to allocate object.
  //
  blockindexing::ptr blockindexing::construct() {
    return blockindexing::ptr(new blockindexing);
  }

  void blockindexing::allocate(const std::set<nodeaddress::ptr, nodeaddress::nodeaddresscomp> &nodes,
                               int ppn,
                               std::vector<nodeaddress::ptr> &result,
                               int nproc)
  {
	  if(nproc < 0){
		  nproc = nodes.size() * ppn;
	  }
    if(nodes.empty()) {
      throw valueerror("blockindexing::allocate:  Empty node vector.");
    }
    if(ppn <= 0) {
      throw valueerror("blockindexing::allocate:  ppn must be > 0.");
    }
    if(ppn*nodes.size()<nproc) {
      throw valueerror("blockindexing::allocate:  require ppn*nodes.size()>=nproc");
    }
    result.clear();
    std::set<nodeaddress::ptr, nodeaddress::nodeaddresscomp>::const_iterator it, end = nodes.end();
    long i = 0;
    for(it = nodes.begin(); it != end && i < nproc / ppn; ++it) {
        SSTMAC_DEBUG << "blockindexing: considering nodeaddress " << (*it)->to_string() << "\n";
      int64_t this_ppn;
      if (result.size() + ppn > nproc) this_ppn = nproc - result.size();
      else this_ppn = ppn;
      result.insert(result.end(), this_ppn, *it);
      i++;
    }
  }

} } // end of namespace sstmac
