/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_INDEXSTRATEGY_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_INDEXSTRATEGY_H_INCLUDED

#include <sstmac/common/nodeaddress.h>
#include <sstmac/common/debug.h>
#include <vector>
#include <set>

namespace sstmac  { namespace sw
  {

  /**
   * Base class for strategies regarding how to sequentially number nodes
   * in a parallel simulation.  
   */
  class indexstrategy : public ptr_type {
  protected:
    static debug dbg_;

    indexstrategy() {
      if(!dbg_.is_real()){
          dbg_ = debug("<debug> indexing | indexstrategy");
          logger::register_logger(&dbg_);
      }
    }
  public:
    virtual std::string to_string() const{
      return "indexstrat";
    }

    typedef boost::intrusive_ptr<indexstrategy> ptr;

    virtual ~indexstrategy() throw() {}

    /// Assign processes to nodes.
    /// \param nodes is the list of unique nodes to be used for the allocation
    /// \param ppn is the nominal number of processes allocated on each node.
    /// \param result is the resulting vector of length nodes (size nproc)
    /// \param nproc the total number of processes to allocate
    virtual void allocate(const std::set<nodeaddress::ptr, nodeaddress::nodeaddresscomp> &nodes,
        int ppn,
        std::vector<nodeaddress::ptr> &result,
        int nproc) = 0;
  };

  } } // end of namespace sstmac

#endif
