/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_BACKENDS_NATIVE_LAUNCH_launchinfo_H_INCLUDED
#define SSTMAC_BACKENDS_NATIVE_LAUNCH_launchinfo_H_INCLUDED

#include <sstmac/common/ptr_type.h>
#include <sstmac/common/timestamp.h>
#include <list>
#include <sstmac/software/process/appid.h>

#include <sstmac/software/skeletons/skeletonfactory.h>

#include <sstmac/common/sim_parameters.h>

namespace sstmac
{
  namespace sw
  {

    class launchinfo : public ptr_type
    {
    public:
      typedef boost::intrusive_ptr<const launchinfo> ptr;
        
      

      timestamp start_;

      app::ptr apptype_;

      appid aid_;

      long num_tasks_;
      
      long ntask_per_node_;

      long extra_nodes_;

      const sim_parameters::ptr params_;

    protected:
      launchinfo(app::ptr app, appid a, const sim_parameters::ptr &p,
          timestamp start, long num_tasks, long ntask_per_node, long extra) :
          apptype_(app), aid_(a), params_(p), start_(start),
          num_tasks_(num_tasks), ntask_per_node_(ntask_per_node), extra_nodes_(extra)
      {

      }



    public:
      static ptr
      construct(std::string t, appid a, sim_parameters::ptr &p,
          timestamp start, long num_tasks, long ntask_per_node, long extra)
      {
        skeletonfactory::init_skeletons(p);
        if(skeletonfactory::apps.find(t) == skeletonfactory::apps.end()){
            std::cout << "Looking for: " << t << std::endl;

            skeletonfactory::print_factory();

            std::cerr << "Could not find application " << t <<". Did you register the class descriptor with "
                      << " the SSTRegisterApp macro?  Is it spelled correctly?" << std::endl;
            throw valueerror("launchinfo::construct - app not found in skeleton factory");
        }
        app::ptr theapp = skeletonfactory::apps[t];
        return ptr(new launchinfo(theapp, a, p, start,
                                  num_tasks, ntask_per_node, extra));
      }

      static ptr
      construct(app::ptr app, appid a, const sim_parameters::ptr &p,
          timestamp start, long num_tasks, long ntask_per_node, long extra)
      {
        return ptr(new launchinfo(app, a, p, start,
                                  num_tasks, ntask_per_node, extra));
      }

      virtual std::string
      to_string() const {
        return "launchinfo";
      }

      virtual ~launchinfo() {}
    };

  }
}
#endif
