/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMACRO_SERVDESCR_H
#define SSTMACRO_SERVDESCR_H

#include <sstmac/software/services/service.h>

#define SSTRegisterServ(servstr, appname) static sstmac::sw::SSTServDesc<appname> appname##_cd(servstr);

namespace sstmac
{
  namespace hw
  {

    class node;
  }
}
namespace sstmac
{
  namespace sw
  {

    class SSTServDesc_base
    {

    public:
      typedef boost::intrusive_ptr<SSTServDesc_base> ptr;

      virtual service::ptr
      build_service() = 0;

    };

    class SSTServDesc_build
    {
    private:
      static std::map<const char*, SSTServDesc_base*>* servmap_;

    public:
      static void
      register_service(const char* name, SSTServDesc_base* descr);

      static void
      start_services(boost::intrusive_ptr<sstmac::hw::node> n);

    };

    template<class T>
      class SSTServDesc : public SSTServDesc_base
      {

      public:
        SSTServDesc(const char* name)
        {
          /** This should only ever be declared within
           the static initialization routine. There is therefore
           nothing unsafe about passing "this" out.  This object
           will be persisent throughout the entire simulation */
          SSTServDesc_build::register_service(name, this);
        }

        service::ptr
        build_service()
        {

          service::ptr serv(T::construct());
          return serv;
        }

      };

  } //end namespace sstmac
} //end namespace sw

#endif

