/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#ifndef SSTMAC_SOFTWARE_SERVICES_SERVICE_H_INCLUDED
#define SSTMAC_SOFTWARE_SERVICES_SERVICE_H_INCLUDED

#include <sstmac/software/process/waiter.h>
#include <sstmac/software/process/operatingsystem.h>

#include <sstmac/software/libraries/library.h>

namespace sstmac
{
  namespace sw
  {

    class service : public library, public eventhandler
    {

    protected:
      service()
      {
        self_ = ptr(this);
        wait_ = waiter::construct();

        init_handler(self_);
        init_waiter(wait_);
      }

    public:
      typedef boost::intrusive_ptr<service> ptr;

      virtual std::string
      to_string() const
      {
        return "service";
      }

      virtual void
      start()
      {

      }

      virtual service::ptr
      clone() = 0;

      virtual void
      init_full_os(const operatingsystem::ptr &o)
      {
        fullos_ = o;
      }

      virtual void
      incoming_message(const sst_message::ptr &msg)
      {
        handle(msg);
      }
    protected:

      operatingsystem::ptr fullos_;

      bool
      is_app_init(appid aid)
      {
        return (fullos_->env_.find(aid) == fullos_->env_.end());
      }

      ptr self_;
      waiter::ptr wait_;

    };

  }
} //end of namespace sstmac

#endif
