/*
 *  This file is part of SST/macroscale:
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top
 *  SST/macroscale directory.
 */

#include <sstmac/software/skeletons/test/app_helloworld.h>

namespace sstmac { namespace sw
{
  using namespace std;
  

  app_helloworld::app_helloworld(softwareid id) :
    app(id)
  {


  }

  // Build a new minimd object.
  //
  app_helloworld::ptr
  app_helloworld::construct(softwareid id)
  {
    app_helloworld::ptr retval(new app_helloworld(id));
    // retval->set_self(retval);
    return retval;
  }

  void
  app_helloworld::consume_params(const sim_parameters::ptr &params)
  {
    whatToSay = params->get_param("helloworld_what");
  }

  //
  // Get a copy of this object.
  //
  app::ptr
  app_helloworld::clone(softwareid id)
  {
    return construct(id);
  }

  app_helloworld::~app_helloworld()
  {

  }

  void
  app_helloworld::skeleton_main()
  {
    SSTMAC_DEBUG << "t=" << now() << ": Hello World! AppID = " << id_.app_.id
          << ", my id = " << id_.task_.id << ", I'm located at "
          << get_physical_address()->to_string()
          << ", and I'm supposed to say: " << whatToSay << ".  I'll compute for a little!\n";

    timestamp t(1e-6);
    compute(t);

    SSTMAC_DEBUG <<"t=" << now() << "It's me again! (" << id_.task_.id << ") - I'm all done! \n";

  }

} } //end of namespace sstmac
