/*
 *  This file is part of SST/macroscale: 
 *               The macroscale architecture simulator from the SST suite.
 *  Copyright (c) 2009 Sandia Corporation.
 *  This software is distributed under the BSD License.
 *  Under the terms of Contract DE-AC04-94AL85000 with Sandia Corporation,
 *  the U.S. Government retains certain rights in this software.
 *  For more information, see the LICENSE file in the top 
 *  SST/macroscale directory.
 */

//#include <sstmac/backends/native/launch/fixedindexing.h>
//#include <sstmac/backends/native/launch/allocationstrategy.h>

#include <sstmac/software/skeletons/undumpi/parsedumpi.h>
#include <sstmac/software/skeletons/undumpi/parsedumpi-callbacks.h>
#include <sstmac/software/skeletons/undumpi/dumpimeta.h>
#include <sstmac/common/errors.h>
#include <boost/lexical_cast.hpp>
#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <fstream>
#include <algorithm>

#include <sstmac/software/services/launch/indexing/fixedindexing.h>
#include <sstmac/software/skeletons/undumpi/dumpiutil.h>

namespace sstmac { namespace sw {


  using namespace std;
  using namespace sstmac::hw;

  //
  // Hi there.
  //
  parsedumpi::parsedumpi(softwareid id) : mpiapp(id, to_string())

  {}
  
  //
  // Good news everyone!  We are getting a new parsedumpi.
  //
  parsedumpi::ptr
  parsedumpi::construct(softwareid id) {
    parsedumpi::ptr retval(new parsedumpi(id));

    return retval;
  }



  void parsedumpi::consume_params(const sim_parameters::ptr &params) {

    std::string index_check = params->get_param("launch_indexing");
    if (index_check != "dumpi")
        throw inputerror("parsedumpi should only be used with dumpi launch indexer");
    std::string alloc_check = params->get_param("launch_allocation");
    if (alloc_check != "dumpi")
        throw inputerror("parsedumpi should only be used with dumpi launch allocation");

    fileroot_ = params->get_param("launch_dumpi_metaname");

    if(params->has_param("parsedumpi_timescale")){

        timescaling_ = params->get_double_param("parsedumpi_timescale");
    }else{
        timescaling_ = 1;
    }

  }

  //
  // Wait!  That's not good news at all!
  //
  parsedumpi::~parsedumpi() throw() {
  }

  //
  // IT'S AN ADDAMS!
  //
  app::ptr parsedumpi::clone(softwareid newid) {
    // put if to check fileroot
    return construct(newid);
  }




  //
  // Parse the file please (this needs an update, there is no reason
  // for it to return a TRACEREC (which should not be uppercase).
  // Need to figure out how to give the caller access to header/footer recs.
  //
  void parsedumpi::skeleton_main() {
    mpi()->init();

   // if((mpi() == 0) || (world().rank().id < 0))
   //   throw illformederror("parsedumpi::run:  Dumpi parser not initialized.");
    dumpimeta::ptr meta = dumpimeta::construct(fileroot_);
    callbacks cbacks(this);
    std::string fname = dumpi_file_name(world().rank().id, meta->dirplusfileprefix_);
    // Ready to go.
    cbacks.parse_stream(fname.c_str());
  }


} } // end of namespace sstmac

