#include <sstmac/sstmacro.h>
#include "sdma_app.h"
#include <mpi/sstmac_mpi.h>

#include "sdma_disk_lib.h"
#include "sdma_staged_lib.h"

using namespace sstmac;
using namespace sstmac::sw;

namespace sdma
{
  SSTRegisterApp("sdma_app", sdma_app)
  ;

  void
  sdma_app::consume_params(const sim_parameters::ptr &params)
  {
    sdma_type_ = params->get_param("sdma_app_system_type");
    numiter_ = params->get_long_param("sdma_app_iterations");
    data_size_ = params->get_long_param("sdma_app_size");
    sleep_ = params->get_time_param("sdma_app_sleep");

    stages_ = params->get_int_param("sdma_app_system_stages");
    stage_conc_ = params->get_double_param("sdma_app_system_concentration");
  }

  void
  sdma_app::init_sdma_system()
  {
    if (sdma_type_ == "disk_pipeline")
      {
        SSTMAC_DEBUG << "sdma_app: using disk system \n";
        sdma_ = sdma_disk_lib::construct(id_);
      }
    else if (sdma_type_ == "staged_pipeline")
      {
        SSTMAC_DEBUG << "sdma_app: using staged system \n";
        sdma_ = sdma_staged_lib::construct(id_);
      }
    else
      {
        std::cout << "system type: " << sdma_type_ << "\n";
        throw ssterror("sdma_app::invalid system type");
      }

    register_lib(sdma_);
    sdma_->init_sdma(stages_, stage_conc_);
  }

  void
  sdma_app::skeleton_main()
  {
    int rank, size;

    MPI_Init(NULL, NULL);
    MPI_Comm_rank(MPI_COMM_WORLD, &rank);

    init_sdma_system();


    for(long i = 0; i < numiter_; i++){
        SSTMAC_DEBUG << "sdma_app[" << rank << "]: starting iteration " << i << "out of " << numiter_ << "\n";

        sdma_->process_data(data_size_);

        sleep(sleep_);
    }

    MPI_Finalize();
  }

}
