#include <sstmac/sstmacro.h>

namespace sendrecv
{

    class sendrecv_skeleton : public sstmac::sw::mpiapp
    {
    public:
      typedef boost::intrusive_ptr<sendrecv_skeleton> ptr;

      typedef boost::intrusive_ptr<const sendrecv_skeleton> const_ptr;

    private:
      int message_size_;

      /// Convenience method.
      const mpicomm&
      world()
      {
        return mpi()->comm_world();
      }

    protected:
      /** Constructor - pass along arguments to parent mpi class */
      sendrecv_skeleton(softwareid id)
          : mpiapp(id, "sendrecv")
      {
      }

    public:
      /** Destructor - do nothing */
      ~sendrecv_skeleton() throw () {}

      static sendrecv_skeleton::ptr
      construct(softwareid id) {
          return new sendrecv_skeleton(id);
      }

      app::ptr
      clone(softwareid newid) {
          return construct(newid);
      }

      void consume_params(const sim_parameters::ptr &params) {
          message_size_ = params->get_int_param("sendrecv_message_size");
      }

      std::string to_string() const {
        return "sendrecv";
      }

      void skeleton_main() {

          mpitag tag(0);
          mpiid dst(1);
          mpiid src(0);

          mpi()->init();
          int me = world().rank().id;
          int nproc = world().size().id;
          if (me == 0 && nproc != 2)
              throw inputerror("sendrecv should only be run with two processors");

          if (me == 0){
              mpi()->send(message_size_, mpitype::mpi_int, dst, tag, world());
              std::cout << "rank " << me << " sending a message\n";
          }
          else {
              mpistatus::const_ptr stat = mpistatus::construct();
              mpi()->recv(message_size_, mpitype::mpi_int,
                          src, tag, world(), stat);
              std::cout << "rank " << me << " receiving a message\n";
          }
          mpi()->finalize();
      }


    };

    SSTRegisterApp("sendrecv", sendrecv_skeleton);
} //end namespace sendrecv



